@@ -51,12 +51,9 @@ void bearing_fill_hbox(GtkWidget *hbox,
 
   if(!isnan(pos.lat) && !isnan(pos.lon)) {
     gtk_box_pack_start(GTK_BOX(hbox), gtk_image_new_from_pixbuf(
-			       		icon_bearing(refpos, pos)),TRUE,FALSE,0);
+	       		icon_bearing(refpos, pos)),TRUE,FALSE,0);
 
     if(!isnan(refpos.lat) && !isnan(refpos.lon)) {
-      gtk_box_pack_start_defaults(GTK_BOX(hbox), 
-		   GTK_LABEL_SMALL((char*)pos_get_bearing_str(refpos, pos)));
-      gtk_box_pack_start_defaults(GTK_BOX(hbox), GTK_LABEL_SMALL("  "));
       snprintf(str, sizeof(str), _("%.1f°"), 
 	       gpx_pos_get_bearing(refpos, pos));
       gtk_box_pack_start_defaults(GTK_BOX(hbox), GTK_LABEL_SMALL(str));
@@ -64,15 +61,15 @@ void bearing_fill_hbox(GtkWidget *hbox,
 			     refpos, pos, appdata->imperial);
       gtk_box_pack_start_defaults(GTK_BOX(hbox), GTK_LABEL_SMALL("  "));
       gtk_box_pack_start(GTK_BOX(hbox), 
-	     gtk_label_attrib(str, SIZE_SMALL, STRIKETHROUGH_NONE),TRUE,FALSE,0);
+          gtk_label_attrib(str, SIZE_SMALL, STRIKETHROUGH_NONE),TRUE,FALSE,0);
     } else
       gtk_box_pack_start(GTK_BOX(hbox), 
-			 gtk_label_attrib(_("(no position)"), 
-			 SIZE_SMALL, STRIKETHROUGH_NONE),TRUE,FALSE,0);
+		 gtk_label_attrib(_("(no position)"), 
+		 SIZE_SMALL, STRIKETHROUGH_NONE),TRUE,FALSE,0);
   } else
     gtk_box_pack_start(GTK_BOX(hbox), 
-	   gtk_label_attrib(_("(invalid position in notes)"), 
-			    SIZE_SMALL, STRIKETHROUGH_NONE),TRUE,FALSE,0);
+		       gtk_label_attrib(_("(invalid position in notes)"), 
+		SIZE_SMALL, STRIKETHROUGH_NONE),TRUE,FALSE,0);
 }
 
 /* this function sets everything related to the coordinate. used to */
@@ -80,48 +77,47 @@ void bearing_fill_hbox(GtkWidget *hbox,
 void overview_coordinate_update(cache_context_t *context) {
   if(!context->notes.modified)
     return;
-
+  
   /* update position labels */
   int strike = notes_get_override(context)?STRIKETHROUGH:STRIKETHROUGH_NONE;
-  char str[32];
-  pos_lat_str(str, sizeof(str), context->cache->pos.lat);
-  gtk_label_attrib_set(context->pos_lat_label, str, SIZE_BIG, strike);
-  pos_lon_str(str, sizeof(str), context->cache->pos.lon);
-  gtk_label_attrib_set(context->pos_lon_label, str, SIZE_BIG, strike);
-
+  lat_label_attrib_set(context->pos_lat_label, 
+		       context->cache->pos.lat, SIZE_BIG, strike);
+  lon_label_attrib_set(context->pos_lon_label, 
+		       context->cache->pos.lon, SIZE_BIG, strike);
+  
   /* remove enire hbox and build a new one */
   gtk_container_foreach(GTK_CONTAINER(context->bearing_hbox), 
 			(GtkCallback)gtk_widget_destroy, NULL);
-
+  
   /* update distance/etc */
   if(!isnan(context->cache->pos.lat) && 
      !isnan(context->cache->pos.lon)) 
     bearing_fill_hbox(context->bearing_hbox, context->appdata, 
 		      *get_pos(context->appdata), notes_get_pos(context)); 
-
+  
   gtk_widget_show_all(context->bearing_hbox);
 }
 
 static void gcvote_set(cache_context_t *context, vote_t *vote) {
   if(!vote) return;
-
+  
   if(context->quality) {
     gtk_widget_destroy(context->quality);
     context->quality = NULL;
   } 
-
+  
   if(context->votes) {
     gtk_widget_destroy(context->votes);
     context->votes = NULL;
   } 
-
+  
   /* update/draw the voting */
 #ifndef USE_MAEMO
   GtkTooltips *tips = gtk_tooltips_new();
 #endif
-
+  
   char *votes_str = g_strdup_printf(_("Quality (%d %s):"), vote->votes,
-				      (vote->votes == 1)?_("vote"):_("votes"));
+				    (vote->votes == 1)?_("vote"):_("votes"));
   context->votes = GTK_LABEL_SMALL(votes_str);
   gtk_box_pack_start(GTK_BOX(context->votebox), 
 		     context->votes, FALSE, FALSE, 0);
@@ -129,7 +125,7 @@ static void gcvote_set(cache_context_t *context, vote_t *vote) {
   context->quality = icon_get_widget(ICON_STARS, (int)(vote->quality*2-2));
   gtk_box_pack_start(GTK_BOX(context->votebox), context->quality,
 		     FALSE, FALSE, 0);
-    
+  
 #ifndef USE_MAEMO
   char *str = g_strdup_printf(_("Quality: %d"), vote->quality);
   gtk_tooltips_set_tip(tips, context->votebox, str, NULL);
@@ -143,11 +139,11 @@ static void gcvote_set(cache_context_t *context, vote_t *vote) {
 
 static void gcvote_callback(vote_t *vote, gpointer data) {
   cache_context_t *context = (cache_context_t*)data;
-
+  
   /* no vote returned: request failed, just cleanup */
   if(!vote) {
     printf("gcvote: callback for failed request\n"); 
-
+    
     gcvote_request_free(context->gcvote_request);
     context->gcvote_request = NULL;
     return;
@@ -157,9 +153,9 @@ static void gcvote_callback(vote_t *vote, gpointer data) {
 	 vote->quality, vote->votes);
   
   gcvote_set(context, vote);
-
+  
   gcvote_save(context->appdata, context->cache, &context->gcvote_request->mem);
-
+  
   gcvote_request_free(context->gcvote_request);
   context->gcvote_request = NULL;
 }
@@ -170,6 +166,12 @@ static void pack_vcentered(GtkWidget *vbox, GtkWidget *child) {
   gtk_box_pack_start_defaults(GTK_BOX(vbox), align);
 }
 
+#ifdef FREMANTLE
+#define ICON_SIZE  ICON_CACHE_TYPE_2X
+#else
+#define ICON_SIZE  ICON_CACHE_TYPE_1_5X
+#endif
+
 static GtkWidget *cache_overview(cache_context_t *context) {
   GtkWidget *vbox, *ivbox;
   GtkWidget *colbox, *tip;
@@ -179,29 +181,24 @@ static GtkWidget *cache_overview(cache_context_t *context) {
 #endif
   appdata_t *appdata = context->appdata;
   cache_t *cache = context->cache;
-
+  
   vbox = gtk_vbox_new(FALSE, 4);
-
+  
   /* hbox containing the four columns of cache details */
   GtkWidget *ihbox = gtk_hbox_new(FALSE, 0);
-
+  
   /* vbox containing leftmost column (icon and container) */
   colbox = gtk_vbox_new(FALSE, 0);
-
+  
   if(cache->type != CACHE_TYPE_UNKNOWN) {
     gtk_box_pack_start_defaults(GTK_BOX(colbox), 
-
-#if defined(USE_MAEMO) && (MAEMO_VERSION_MAJOR >= 5)
-    tip = icon_get_widget(ICON_CACHE_TYPE_2X, cache->type));
-#else
-    tip = icon_get_widget(ICON_CACHE_TYPE_1_5X, cache->type));
-#endif
-
+	tip = icon_get_widget(ICON_SIZE, cache->type));
+    
 #ifndef USE_MAEMO
     gtk_tooltips_set_tip(tips, tip, _(cache_type_tip[cache->type]), NULL);
 #endif
   }
-
+  
   /* ------------ box containing container info ------------ */
   if(cache->container != CACHE_CONT_UNKNOWN) {
     ivbox = gtk_vbox_new(FALSE, 0);
@@ -211,16 +208,16 @@ static GtkWidget *cache_overview(cache_context_t *context) {
     gtk_box_pack_start(GTK_BOX(ivbox), 
 		       icon_get_widget(ICON_CACHE_SIZE, cache->container),
 		       FALSE, FALSE, 0);
-
+    
     pack_vcentered(colbox, ivbox);
 #ifndef USE_MAEMO
     gtk_tooltips_set_tip(tips, ivbox, _(cache_size_tip[cache->container]), NULL);
 #endif
   }
-
+  
   gtk_box_pack_start_defaults(GTK_BOX(ihbox), colbox);
   colbox = gtk_vbox_new(FALSE, 0);
-
+  
   /* ----------------------- id ---------------------------- */
   if(cache->id) {
     int strike = cache->archived?STRIKETHROUGH_RED:
@@ -599,8 +596,13 @@ static GtkWidget *cache_logs(appdata_t *appdata, cache_context_t *context,
 
 #ifdef ENABLE_BROWSER_INTERFACE
   if(gc_link) {
-    GtkWidget *but = 
+    GtkWidget *but =
       gtk_button_new_with_label(_("Post a new log entry for this geocache"));
+#if 0
+    gtk_button_set_image(GTK_BUTTON(but), 
+	gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_BUTTON));
+    gtk_button_set_image_position(GTK_BUTTON(but), GTK_POS_LEFT);
+#endif
 #if defined(USE_MAEMO) && (MAEMO_VERSION_MAJOR == 5)
   hildon_gtk_widget_set_theme_size(but, 
 	   (HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH));
@@ -635,7 +637,7 @@ static GtkWidget *cache_logs(appdata_t *appdata, cache_context_t *context,
 		   (HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH));
 #endif  
 
-      gtk_label_set_ellipsize(GTK_LABEL(gtk_bin_get_child(GTK_BIN(finder))), 
+    gtk_label_set_ellipsize(GTK_LABEL(gtk_bin_get_child(GTK_BIN(finder))), 
 			      PANGO_ELLIPSIZE_END);
     } else
       gtk_label_set_ellipsize(GTK_LABEL(finder), PANGO_ELLIPSIZE_END);
@@ -726,13 +728,12 @@ static void on_notebook_page_change(GtkNotebook     *notebook,
   /* seems to work ... */
   if(strcasecmp(name, _("Logs")) == 0) {
     gtk_widget_queue_resize(w);
-  } else if(strcasecmp(name, _("TBs")) == 0) {
-    gtk_widget_queue_resize(w);
   } else if(strcasecmp(name, _("Goto")) == 0) {
 #ifdef USE_MAEMO  
     context->handler_id = gtk_timeout_add(1000, screensaver_update, 
 					  context->appdata);
 #endif
+
     goto_coordinate_update(context);
   }
 
@@ -762,7 +763,19 @@ static void on_notebook_destroy(GtkWidget *widget, gpointer user_data ) {
     gtk_timeout_remove(context->handler_id);
 #endif
 
-  free(user_data);
+#ifdef USE_STACKABLE_WINDOW
+  if(context->notes_have_been_changed) {
+    printf("notes changed -> cachelist redraw\n");
+
+    /* now the cachelist is visible again. so redraw it since it may */
+    /* have changed */
+    cachelist_redraw(context->appdata);
+  }
+#endif
+
+  printf("freeing cache context\n");
+  context->appdata->cache_context = NULL;
+  g_free(context);
 }
 
 #if defined(USE_MAEMO) && (MAEMO_VERSION_MAJOR >= 5)
@@ -862,8 +875,8 @@ static GObject *notebook_object(GtkWidget *notebook) {
 GtkWidget *cache_view(appdata_t *appdata, cache_t *cache) {
   GtkWidget *notebook;
 
-  cache_context_t *cache_context = malloc(sizeof(cache_context_t));
-  memset(cache_context, 0, sizeof(cache_context_t));
+  cache_context_t *cache_context = g_new0(cache_context_t, 1);
+  appdata->cache_context = cache_context;
   cache_context->appdata = appdata;
   cache_context->cache = cache;
 
@@ -941,20 +954,22 @@ void cache_dialog(appdata_t *appdata, cache_t *cache) {
 static void on_cache_destroy (GtkWidget *widget, appdata_t *appdata) {
   appdata->cur_cache = NULL;
 
+  HildonWindowStack *stack = hildon_window_stack_get_default();
+  appdata->window = HILDON_WINDOW(hildon_window_stack_peek(stack));
+
   /* restore cur_view */
   appdata->cur_view = g_object_get_data(G_OBJECT(widget), "cur_view");
 }
 
 void cache_dialog(appdata_t *appdata, cache_t *cache) {
   GtkWidget *window = hildon_stackable_window_new();
+  appdata->window = HILDON_WINDOW(window);
 
   /* store last "cur_view" in window */
   g_object_set_data(G_OBJECT(window), "cur_view", appdata->cur_view);
 
   appdata->cur_cache = cache;
-  char *title = g_strdup_printf("%s - GPXView", cache->name);
-  gtk_window_set_title(GTK_WINDOW(window), title);
-  g_free(title);
+  gtk_window_set_title(GTK_WINDOW(window), cache->name);
 
   /* create cache visualization widget */
   appdata->cur_view = cache_view(appdata, cache);
