@@ -192,6 +192,7 @@ void geotext_dialog(appdata_t *appdata) {
 
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), 
 		     hbox, FALSE, FALSE, 0);
+
 #endif
 
 #ifndef USE_PANNABLE_AREA
@@ -250,7 +251,7 @@ void geotext_dialog(appdata_t *appdata) {
 		     (GtkSignalFunc)on_left_shift_clicked, &context);
   gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
 
-  context.entry = gtk_entry_new();
+  context.entry = entry_new();
   char str[32];
   snprintf(str, sizeof(str), "%d", appdata->geotext_shift);
   gtk_entry_set_text(GTK_ENTRY(context.entry), str);
