@@ -182,14 +182,7 @@ void gps_clear_fix(struct gps_fix_t *fixp) {
   fixp->mode = MODE_NOT_SEEN;
   fixp->pos.lat = fixp->pos.lon = NAN;
   fixp->track = NAN;
-  fixp->speed = NAN;
-  fixp->climb = NAN;
-  fixp->altitude = NAN;
-  fixp->ept = NAN;
   fixp->eph = NAN;
-  fixp->epd = NAN;
-  fixp->eps = NAN;
-  fixp->epc = NAN;
 }
 
 /* unpack a daemon response into a status structure */
@@ -205,14 +198,7 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata) {
 	else *tp = '\0';
 	
 	switch (*sp) {
-	case 'A':
-	  if (sp[2] == '?') {
-	    gpsdata->fix.altitude = NAN;
-	  } else {
-	    (void)sscanf(sp, "A=%lf", &gpsdata->fix.altitude);
-	    gpsdata->set |= ALTITUDE_SET;
-	  }
-	  break;
+	  /* A - altitude is not supported */
 	  /* B - baudrate isn't supported by gpxview */
 	  /* C - cycle isn't supported by gpxview */
 	  /* D - utc time isn't supported by gpxview */
@@ -261,32 +247,19 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata) {
 #define DEFAULT(val) (val[0] == '?') ? NAN : g_ascii_strtod(val, NULL)
 	      nf.pos.lat = DEFAULT(lat);
 	      nf.pos.lon = DEFAULT(lon);
-	      nf.ept = DEFAULT(ept);
-	      nf.altitude = DEFAULT(alt);
 	      nf.eph = DEFAULT(eph);
 	      nf.track = DEFAULT(track);
-	      nf.speed = DEFAULT(speed);
-	      nf.climb = DEFAULT(climb);
-	      nf.epd = DEFAULT(epd);
-	      nf.eps = DEFAULT(eps);
-	      nf.epc = DEFAULT(epc);
 #undef DEFAULT
 	      if (st >= 15)
 		nf.mode = (mode[0] == '?') ? MODE_NOT_SEEN : atoi(mode);
 	      else
 		nf.mode = (alt[0] == '?') ? MODE_2D : MODE_3D;
-	      if (alt[0] != '?')
-		gpsdata->set |= ALTITUDE_SET | CLIMB_SET;
 	      if (isnan(nf.eph)==0)
 		gpsdata->set |= HERR_SET;
 	      if (isnan(nf.track)==0)
-		gpsdata->set |= TRACK_SET | SPEED_SET;
-	      if (isnan(nf.eps)==0)
-		gpsdata->set |= SPEEDERR_SET;
-	      if (isnan(nf.epc)==0)
-		gpsdata->set |= CLIMBERR_SET;
+		gpsdata->set |= TRACK_SET;
 	      gpsdata->fix = nf;
-	      gpsdata->set |= TIME_SET|TIMERR_SET|LATLON_SET|MODE_SET;
+	      gpsdata->set |= LATLON_SET|MODE_SET;
 	      gpsdata->status = STATUS_FIX;
 	      gpsdata->set |= STATUS_SET;
 	    }
@@ -304,23 +277,7 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata) {
 	    gpsdata->set |= LATLON_SET;
 	  }
 	  break;
-	case 'Q':
-	  if (sp[2] == '?') {
-	    gpsdata->satellites_used = 0;
-	    gpsdata->pdop = 0;
-	    gpsdata->hdop = 0;
-	    gpsdata->vdop = 0;
-	  } else {
-	    (void)sscanf(sp, "Q=%d %lf %lf %lf %lf %lf",
-			 &gpsdata->satellites_used,
-			 &gpsdata->pdop,
-			 &gpsdata->hdop,
-			 &gpsdata->vdop,
-			 &gpsdata->tdop,
-			 &gpsdata->gdop);
-	    gpsdata->set |= HDOP_SET | VDOP_SET | PDOP_SET;
-	  }
-	  break;
+	  /* Q is not supported */
 	case 'S':
 	  if (sp[2] == '?') {
 	    gpsdata->status = -1;
@@ -333,36 +290,13 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata) {
 	  if (sp[2] == '?') {
 	    gpsdata->fix.track = NAN;
 	  } else {
-	    (void)sscanf(sp, "T=%lf", &gpsdata->fix.track);
+	    (void)sscanf(sp, "T=%f", &gpsdata->fix.track);
 	    gpsdata->set |= TRACK_SET;
 	  }
 	  break;
-	case 'U':
-	  if (sp[2] == '?') {
-	    gpsdata->fix.climb = NAN;
-	  } else {
-	    (void)sscanf(sp, "U=%lf", &gpsdata->fix.climb);
-	    gpsdata->set |= CLIMB_SET;
-	  }
-	  break;
-	case 'V':
-	  if (sp[2] == '?') {
-	    gpsdata->fix.speed = NAN;
-	  } else {
-	    (void)sscanf(sp, "V=%lf", &gpsdata->fix.speed);
-	    /* V reply is in kt, fix.speed is in metres/sec */
-	    gpsdata->fix.speed = gpsdata->fix.speed / MPS_TO_KNOTS;
-	    gpsdata->set |= SPEED_SET;
-	  }
-	  break;
-	case 'X':
-	  if (sp[2] == '?') 
-	    gpsdata->online = -1;
-	  else {
-	    (void)sscanf(sp, "X=%lf", &gpsdata->online);
-	    gpsdata->set |= ONLINE_SET;
-	  }
-	  break;
+	  /* U - climb is not supported */
+	  /* V - is not supported */
+	  /* X - online is not supported */
 	case 'Y':
 	  if (sp[2] == '?') {
 	    gpsdata->satellites = 0;
@@ -416,7 +350,7 @@ static void do_app_cb(gpointer data, gpointer user_data) {
 /* walk though list of all application provided callbacks */
 static gboolean gps_idle_cb(gpointer data) {
   appdata_t *appdata = (appdata_t*)data;
-  printf("gps: idle callback, calling app callbacks\n");
+  //  printf("gps: idle callback, calling app callbacks\n");
 
   g_slist_foreach(appdata->gps_state->cb, do_app_cb, appdata);
 
@@ -472,7 +406,7 @@ gpointer gps_thread(gpointer data) {
 	      
 	      if(!cnt) appdata->gps_state->gpsdata.set &= ~SATELLITE_SET;
 	      else     appdata->gps_state->gpsdata.set &= 
-			 ~(TIME_SET|TIMERR_SET|LATLON_SET|MODE_SET|STATUS_SET);
+			 ~(LATLON_SET|MODE_SET|STATUS_SET);
 	    
 	      gps_unpack(str, &appdata->gps_state->gpsdata);
 	      g_mutex_unlock(appdata->gps_state->mutex);
@@ -525,13 +459,13 @@ location_changed(LocationGPSDevice *device, gps_state_t *gps_state) {
   gps_state->fields = device->fix->fields;
 
   if(gps_state->fields & LOCATION_GPS_DEVICE_LATLONG_SET) { 
-    gps_state->latitude = device->fix->latitude;
-    gps_state->longitude = device->fix->longitude;
-    gps_state->eph = device->fix->eph/100.0;  // we want eph in meters
+    gps_state->fix.pos.lat = device->fix->latitude;
+    gps_state->fix.pos.lon = device->fix->longitude;
+    gps_state->fix.eph = device->fix->eph/100.0;  // we want eph in meters
   }
 
   if(gps_state->fields & LOCATION_GPS_DEVICE_TRACK_SET)
-    gps_state->heading = device->fix->track;
+    gps_state->fix.track = device->fix->track;
   
   /* update list of sattelites */
 
@@ -624,8 +558,8 @@ pos_t *gps_get_pos(appdata_t *appdata) {
   if(!(gps_state->fields & LOCATION_GPS_DEVICE_LATLONG_SET))
     return NULL;
 
-  pos.lat = gps_state->latitude;
-  pos.lon = gps_state->longitude;
+  pos.lat = gps_state->fix.pos.lat;
+  pos.lon = gps_state->fix.pos.lon;
 
   return &pos; 
 }
@@ -636,7 +570,7 @@ float gps_get_heading(appdata_t *appdata) {
   if(!(gps_state->fields & LOCATION_GPS_DEVICE_TRACK_SET))
     return NAN;
 
-  return gps_state->heading; 
+  return gps_state->fix.track; 
 }
 
 float gps_get_eph(appdata_t *appdata) { 
@@ -645,7 +579,7 @@ float gps_get_eph(appdata_t *appdata) {
   if(!(gps_state->fields & LOCATION_GPS_DEVICE_LATLONG_SET))
     return NAN;
 
-  return gps_state->eph; 
+  return gps_state->fix.eph; 
 }
 
 gps_sat_t *gps_get_sats(appdata_t *appdata) { 
