@@ -24,6 +24,8 @@
 #if MAEMO_VERSION_MAJOR < 5
 #define USE_BREAD_CRUMB_TRAIL
 #else
+#define FREMANTLE
+
 // these are on maemo fremantle
 #include <hildon/hildon-button.h>
 #include <hildon/hildon-text-view.h>
@@ -130,7 +132,6 @@ typedef struct {
   GtkWidget    *gpxview;
   GConfClient *gconf_client;
 
-  pos_t manual_goto;
   gboolean compass_locked;
   int compass_damping;
 
@@ -177,8 +178,10 @@ typedef struct {
 
 #ifdef USE_MAEMO
   gboolean fullscreen;
+#ifdef ENABLE_MAEMO_MAPPER
   pos_t mmpos;                /* position received from Maemo Mapper */
   gboolean mmpos_valid;
+#endif
 
   gboolean mmpoi_dontlaunch;
 
@@ -205,6 +208,7 @@ typedef struct {
   /* keep track of current "path" in the bread crumb trail */
   gpx_t *cur_gpx;
   cache_t *cur_cache;
+  struct cache_context_s *cache_context;
 
   gpx_t *search_results;
 
@@ -217,6 +221,16 @@ typedef struct {
   } map;
 #endif
 
+  /* username is also used to detect own caches (by log entry) */
+  char *username;
+
+  /* stuff to call geotoad */
+  struct {
+    char *password, *filename;
+    float distance, lat, lon;
+    int flags;
+  } gt;
+
   GtkWidget *vbox;    
 } appdata_t;
 
@@ -235,6 +249,8 @@ typedef struct {
 #include "mm_poi.h"
 #include "precpos.h"
 #include "garmin_export.h"
+#include "geotoad.h"
+#include "about.h"
 
 #ifdef ENABLE_OSM_GPS_MAP
 #include "map-tool.h"
@@ -276,5 +292,9 @@ HildonAppMenu *menu_create(appdata_t *appdata, int mode);
 #endif
 void cachelist_goto_cache(appdata_t *appdata, cache_t *cache);
 void gpxlist_goto_cachelist(appdata_t *appdata, gpx_t *gpx);
+gboolean gpxlist_find(appdata_t *appdata, GtkTreeIter *iter, gpx_t *gpx);
+void gpxlist_set(GtkListStore *store, GtkTreeIter *iter, gpx_t *gpx);
+void gpxlist_add(appdata_t *appdata, gpx_t *new);
+void cachelist_redraw(appdata_t *appdata);
 
 #endif // GPXVIEW_H
