@@ -19,6 +19,13 @@
 
 #include "gpxview.h"
 
+// #undef PANNABLE_HTML
+
+#ifdef FREMANTLE
+#include <hildon/hildon-banner.h>
+#include <hildon/hildon-note.h>
+#endif
+
 typedef struct load_context {
   int active;
   GMutex *mutex;
@@ -138,11 +145,38 @@ gpointer loader_thread(gpointer data) {
   return NULL;
 }
 
+#ifdef ENABLE_BROWSER_INTERFACE
+static void on_link_clicked(GtkHTML *html, const gchar *url, 
+			    gpointer data) {
+
+  appdata_t *appdata = (appdata_t*)data;
+
+#if !defined(USE_MAEMO) || (MAEMO_VERSION_MAJOR < 5)
+  GtkWidget *dialog = gtk_message_dialog_new(
+			   GTK_WINDOW(appdata->window),
+			   GTK_DIALOG_DESTROY_WITH_PARENT,
+			   GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, 
+			   _("Open link on external browser?"));
+  gboolean yes = (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_YES);
+#else
+  GtkWidget *dialog = 
+    hildon_note_new_confirmation(GTK_WINDOW(appdata->window), 
+				_("Open link in external browser?"));
+  gboolean yes = (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK);
+#endif
+  
+  gtk_widget_destroy(dialog);
+
+  if(yes)
+    browser_url(appdata, (char*)url);
+}
+#endif
+
 static void on_request_url(GtkHTML *html, const gchar *url, 
 			   GtkHTMLStream *stream, gpointer data) {
   char buffer[4096];
   GnomeVFSFileSize bytes_read;
-  
+
   http_context_t *context = (http_context_t*)data;
 
   if(context->cache) {
@@ -347,11 +381,36 @@ void html_copy_to_clipboard(appdata_t *appdata) {
 }
 #endif
 
-#ifdef PANNABLE_HTML
-/* eat the button events */
-static gboolean on_button_press(GtkWidget *widget, GdkEventButton *event,
-				gpointer user_data) {
-  return TRUE;
+#ifdef FREMANTLE
+static void
+tap_and_hold_cb (GtkWidget *widget, gpointer user_data) {
+  appdata_t *appdata = (appdata_t*)user_data;
+
+  /* the HildonTextView becomes semi-copy'n pasteable if we enable */
+  /* the cursor */
+  if(GTK_WIDGET_TYPE(widget) == HILDON_TYPE_TEXT_VIEW) {
+    gboolean state = gtk_text_view_get_cursor_visible(GTK_TEXT_VIEW(widget));
+    gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(widget), !state);
+    return;
+  }
+
+#if 0
+  GtkWidget *parent = widget->parent;
+
+  /* check if the parent actually is a hildonpannablearea */
+  if(GTK_WIDGET_TYPE(parent) != HILDON_TYPE_PANNABLE_AREA) {
+    printf("parent is not pannable area, ignoring event\n");
+    return;
+  }
+
+  gboolean enabled = FALSE;
+  g_object_get(parent, "enabled", &enabled, NULL);
+  enabled = !enabled;
+  g_object_set(parent, "enabled", enabled, NULL);  
+
+  hildon_banner_show_information(GTK_WIDGET(appdata->window), 
+				 NULL, enabled?"enabled":"disabled");
+#endif
 }
 #endif
 
@@ -373,11 +432,28 @@ GtkWidget *html_view(appdata_t *appdata, char *text,
 
     context->view = view = gtk_html_new();
 
+#ifndef PANNABLE_HTML
+    //    gtk_html_set_auto_panning(GTK_HTML(view), TRUE);
+#else
+    gtk_html_set_auto_panning(GTK_HTML(view), FALSE);
+#endif
+
+#ifdef FREMANTLE
+    /* allow selection does not allow anything, but disables auto panning ... */
+    /* even worse: frequently causes the device to reboot */
+    //    gtk_html_allow_selection(GTK_HTML(view), TRUE);    
+#endif
+
     /* create a callback to load images only if a cache has been given */
     /* so that images can be cached/stored appropriately */
     g_signal_connect(G_OBJECT(view), "url_requested", 
 		     G_CALLBACK(on_request_url), context);
 
+#ifdef ENABLE_BROWSER_INTERFACE
+    g_signal_connect(G_OBJECT(view), "link_clicked", 
+		     G_CALLBACK(on_link_clicked), context->appdata);
+#endif
+
     GtkHTMLStream *stream = gtk_html_begin(GTK_HTML(view));
 
     gtk_html_write(GTK_HTML(view), stream, html_start, strlen(html_start));
@@ -398,13 +474,10 @@ GtkWidget *html_view(appdata_t *appdata, char *text,
     *h = g_new0(struct html_view, 1);
     (*h)->view = view;
 
-#ifdef PANNABLE_HTML
-    /* this causes finger scrolling to work nicely but also prevents */
-    /* copy'n paste from working correctly */
-    gtk_widget_set_sensitive(GTK_WIDGET(view), FALSE);
-
-    g_signal_connect(G_OBJECT(view), "button-press-event", 
-		     G_CALLBACK(on_button_press), NULL);
+#ifdef FREMANTLE
+    gtk_widget_tap_and_hold_setup(GTK_WIDGET(view), NULL, NULL, 0);
+    g_signal_connect(G_OBJECT(view), "tap-and-hold", 
+		     G_CALLBACK(tap_and_hold_cb), appdata);
 #endif
 
     g_signal_connect(G_OBJECT(view), "destroy", 
@@ -418,33 +491,27 @@ GtkWidget *html_view(appdata_t *appdata, char *text,
 #else
     view = hildon_text_view_new();
     hildon_text_view_set_buffer(HILDON_TEXT_VIEW(view), buffer);
+
+    gtk_widget_tap_and_hold_setup(GTK_WIDGET(view), NULL, NULL, 0);
+    g_signal_connect(G_OBJECT(view), "tap-and-hold", 
+		     G_CALLBACK(tap_and_hold_cb), appdata);
 #endif
 
     gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(view), GTK_WRAP_WORD);
     gtk_text_view_set_editable(GTK_TEXT_VIEW(view), FALSE);
     gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(view), FALSE);
 
-    // #if MAEMO_VERSION_MAJOR >= 5
-#if 1
     /* make this look nicer in fremantle and not just black-on-white */
-
-#if 0
-    /* this causes a black background */
-    gtk_widget_modify_base(view, GTK_STATE_NORMAL, 
-	   &GTK_WIDGET(appdata->window)->style->bg[GTK_STATE_NORMAL]);
-    /* and this causes white letters */
-    gtk_widget_modify_text(view, GTK_STATE_NORMAL, 
-	   &GTK_WIDGET(appdata->window)->style->text[GTK_STATE_NORMAL]);
-#else
     /* just use the default style */
-#if 1
-    gtk_widget_set_style(view, GTK_WIDGET(appdata->window)->style);
+#ifdef USE_STACKABLE_WINDOW
+    /* in fremantle this is really tricky and we need to inherit the */
+    /* style from the topmost window in the stack */
+    HildonWindowStack *stack = hildon_window_stack_get_default();
+    GList *list = hildon_window_stack_get_windows(stack);
+    gtk_widget_set_style(view, GTK_WIDGET(list->data)->style);
+    g_list_free(list);
 #else
-    GtkStyle *style = gtk_rc_get_style(GTK_WIDGET(appdata->window));
-    gtk_widget_set_style(view, style);
-#endif
-#endif
-
+    gtk_widget_set_style(view, GTK_WIDGET(appdata->window)->style);
 #endif
 
 #ifndef NO_COPY_N_PASTE
@@ -465,15 +532,7 @@ GtkWidget *html_view(appdata_t *appdata, char *text,
 		     GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
     gtk_container_add(GTK_CONTAINER(scrolled_window), view);
 
-#if 1
     return scrolled_window;
-#else
-    GtkWidget *fixed = gtk_fixed_new();
-    gtk_fixed_put(GTK_FIXED(fixed), scrolled_window, 0, 0);
-    GtkWidget *tbutton = gtk_toggle_button_new_with_label("sel");
-    gtk_fixed_put(GTK_FIXED(fixed), tbutton, 0, 0);
-    return fixed;
-#endif
 #else
 #ifndef PANNABLE_HTML
   if(mode == HTML_HTML) { 
