@@ -24,18 +24,19 @@ struct icon_data {
   GdkPixbuf **data;
   int count;
 } icons[] = {
-  { NULL, CACHE_TYPE_MAX+4 },  /* ICON_CACHE_TYPE      */
-  { NULL, CACHE_TYPE_MAX+4 },  /* ICON_CACHE_TYPE_SEMI */
-  { NULL, CACHE_TYPE_MAX+4 },  /* ICON_CACHE_TYPE_1_5X */
-  { NULL, CACHE_TYPE_MAX+4 },  /* ICON_CACHE_TYPE_2X   */
-  { NULL, CACHE_CONT_MAX+1 },  /* ICON_CACHE_SIZE      */
-  { NULL, 9 },                 /* ICON_STARS           */
-  { NULL, LOG_TYPE_MAX+1 },    /* ICON_LOG             */
-  { NULL, 9 },                 /* ICON_HEADING         */
-  { NULL, WPT_SYM_MAX+1 },     /* ICON_WPT             */
-  { NULL, 2 },                 /* ICON_TB              */
-  { NULL, 6 },                 /* ICON_MISC            */
-  { NULL, 3 },                 /* ICON_FILE            */
+  { NULL, CACHE_TYPE_MAX+5 },             /* ICON_CACHE_TYPE      */
+  { NULL, CACHE_TYPE_MAX+5 },             /* ICON_CACHE_TYPE_SEMI */
+  { NULL, CACHE_TYPE_MAX+5 },             /* ICON_CACHE_TYPE_1_5X */
+  { NULL, CACHE_TYPE_MAX+5 },             /* ICON_CACHE_TYPE_2X   */
+  { NULL, CACHE_CONT_MAX+1 },             /* ICON_CACHE_SIZE      */
+  { NULL, 9 },                            /* ICON_STARS           */
+  { NULL, LOG_TYPE_MAX+1 },               /* ICON_LOG             */
+  { NULL, 9 },                            /* ICON_HEADING         */
+  { NULL, WPT_SYM_MAX+1 },                /* ICON_WPT             */
+  { NULL, 2 },                            /* ICON_TB              */
+  { NULL, 6 },                            /* ICON_MISC            */
+  { NULL, WPT_SYM_MAX+CACHE_TYPE_MAX+10 },/* ICON_POS             */
+  { NULL, 3 },                            /* ICON_FILE            */
   { NULL, -1 }
 };
 
@@ -45,7 +46,7 @@ const char *cache_type_icon_name[] = {
   "letterbox",   "earthcache", "wherigo", "megaevent", "cito",
 
   /* special overlays */
-  "ovl_override", "ovl_found", "ovl_note", 
+  "ovl_override", "ovl_found", "ovl_note", "ovl_mine",
 
   NULL
 };
@@ -90,8 +91,36 @@ const char *tb_icon_name[] = {
 
 /* ICON_MISC         */
 const char *misc_icon_name[] = {
-  "maemo-mapper-out", "locked", "unlocked",
-  "maemo-mapper-in", "delete", "paypal", NULL
+  "locked", "unlocked", "delete", 
+#ifdef FREMANTLE
+  "64x64/paypal", 
+#else
+  "32x32/paypal", 
+#endif
+  "32x32/pos_geomath", 
+#ifdef FREMANTLE
+  "64x64/gpxview",
+#else
+  "32x32/gpxview",
+#endif
+  NULL
+};
+
+/* ICON_POS        */
+const char *pos_icon_name[] = {
+  "wpt_multistage",         "wpt_parking",           "wpt_final",
+  "wpt_question",           "wpt_trailhead",         "wpt_refpoint", 
+
+  "cache_type_traditional", "cache_type_multi",      "cache_type_mystery", 
+  "cache_type_virtual",     "cache_type_webcam",     "cache_type_event",
+  "cache_type_letterbox",   "cache_type_earthcache", "cache_type_wherigo", 
+  "cache_type_megaevent",   "cache_type_cito",
+
+  "pos_wpt", "pos_gps", "pos_geomath", "pos_map", "pos_home",
+
+  "pos_get", "pos_set", "maemo-mapper",
+
+  NULL
 };
 
 /* ICON_FILE         */
@@ -136,7 +165,7 @@ static void icons_load(int type, char *format, const char *names[]) {
       }   
     }
   }
-
+  
   g_assert(!names[i]);
 }
 
@@ -187,6 +216,13 @@ void icons_init(void) {
   /* load misc icons */
   icons_load(ICON_MISC, "%s.%s", misc_icon_name);
 
+  /* load pos icons */
+#if !defined(USE_MAEMO) || (MAEMO_VERSION_MAJOR < 5)
+  icons_load(ICON_POS, "16x16/%s.%s", pos_icon_name);
+#else
+  icons_load(ICON_POS, "32x32/%s.%s", pos_icon_name);
+#endif
+
   /* load file icons */
   icons_load(ICON_FILE, "file_%s.%s", file_icon_name);
 }
