@@ -15,8 +15,10 @@
  * along with GPXView.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <stdio.h>
+#define __USE_GNU
 #include <string.h>
+
+#include <stdio.h>
 #include <math.h>
 
 #include <curl/curl.h>
@@ -34,7 +36,6 @@
 #if MAEMO_VERSION_MAJOR >= 5
 #include <hildon/hildon-note.h>
 #include <hildon/hildon-entry.h>
-#include <hildon/hildon-check-button.h>
 #endif
 #endif
 
@@ -155,15 +156,15 @@ gpx_t *choose_file(appdata_t *appdata, gboolean whole_dir) {
       gpx_dialog_t *dialog = gpx_busy_dialog_new(GTK_WIDGET(appdata->window));
 
       if(!whole_dir) 
-	gpx = gpx_parse(dialog, filename);
+	gpx = gpx_parse(dialog, filename, appdata->username);
       else {
 	/* cur trailing '/' if present */
 	if(strlastchr(filename) == '/')
 	  filename[strlen(filename)] = 0;
 	
-	gpx = gpx_parse_dir(dialog, filename);
+	gpx = gpx_parse_dir(dialog, filename, appdata->username);
       }
-      
+
       gpx_busy_dialog_destroy(dialog);
       
       /* save path if gpx was successfully loaded */
@@ -221,12 +222,12 @@ void cachelist_goto_cache(appdata_t *appdata, cache_t *cache) {
 #if !defined(USE_BREAD_CRUMB_TRAIL) && !defined(BCT)
     cache_dialog(appdata, cache);
 #else
+    crumb_add(appdata, cache->name, CRUMB_CACHE, cache);
+
     gtk_container_remove(GTK_CONTAINER(appdata->vbox), appdata->cur_view);
     appdata->cur_view = cache_view(appdata, cache);
     gtk_box_pack_start_defaults(GTK_BOX(appdata->vbox), appdata->cur_view);
     gtk_widget_show_all(appdata->vbox);
-
-    crumb_add(appdata, cache->name, CRUMB_CACHE, cache);
 #endif
 }
 
@@ -474,10 +475,12 @@ static GtkWidget *cachelist_create(appdata_t *appdata, gpx_t *gpx,
     if(tint > 8) tint = 8;
 
     /* cache type includes "solved" flag in lowest bit */
-    int type = (cache->type<<8) +
-      (cache->notes?4:0) +
-      ((cache->notes && cache->notes->override)?1:0) +
-      ((cache->notes && cache->notes->found)?2:0);
+    int type = cache->type<<8;
+    if(cache->notes) type |= 4;
+    if(cache->notes && cache->notes->override) type |= 1;
+    if(cache->notes && cache->notes->found) type |= 2;
+    if(cache->found) type |= 2;
+    if(cache->mine) type |= 8;
 
     if((!(type & 2)) || !appdata->cachelist_hide_found) {
 
@@ -593,6 +596,11 @@ void cachelist_dialog(appdata_t *appdata, gpx_t *gpx) {
 static void search_result_free(gpx_t *result);
 
 void on_cachelist_destroy(GtkWidget *widget, appdata_t *appdata) {
+  printf("cachelist destroy\n");
+
+  HildonWindowStack *stack = hildon_window_stack_get_default();
+  appdata->window = HILDON_WINDOW(hildon_window_stack_peek(stack));
+
   if(appdata->search_results) {
     search_result_free(appdata->search_results);
     appdata->search_results = NULL;
@@ -609,14 +617,13 @@ void on_cachelist_destroy(GtkWidget *widget, appdata_t *appdata) {
 
 void cachelist_dialog(appdata_t *appdata, gpx_t *gpx) {
   GtkWidget *window = hildon_stackable_window_new();
-
+  appdata->window = HILDON_WINDOW(window);
+  
   /* store last "cur_view" in window */
   g_object_set_data(G_OBJECT(window), "cur_view", appdata->cur_view);
 
   appdata->cur_gpx = gpx;
-  char *title = g_strdup_printf("%s - GPXView", gpx->name);
-  gtk_window_set_title(GTK_WINDOW(window), title);
-  g_free(title);
+  gtk_window_set_title(GTK_WINDOW(window), gpx->name);
 
   appdata->cur_view = cachelist_create(appdata, gpx, NULL);
   gtk_container_add(GTK_CONTAINER(window), appdata->cur_view);
@@ -665,7 +672,7 @@ static GdkPixbuf *gpx_icon_get(gpx_t *gpx) {
   return icon_get(ICON_FILE, 0);
 }
 
-static void gpxlist_set(GtkListStore *store, GtkTreeIter *iter, gpx_t *gpx) {
+void gpxlist_set(GtkListStore *store, GtkTreeIter *iter, gpx_t *gpx) {
   char date_str[32], cnum[32];
 
   if(gpx->year && gpx->month && gpx->day) {
@@ -690,7 +697,7 @@ static void gpxlist_set(GtkListStore *store, GtkTreeIter *iter, gpx_t *gpx) {
 		     GPXLIST_COL_OPEN, !gpx->closed,
 		     GPXLIST_COL_CACHES, gpx->closed?NULL:cnum,
 #ifdef USE_PANNABLE_AREA
-		     GPXLIST_COL_DELETE, icon_get(ICON_MISC, 4),
+		     GPXLIST_COL_DELETE, icon_get(ICON_MISC, 2),
 #endif
 		     GPXLIST_COL_DATA, gpx,
 		     -1);
@@ -864,9 +871,9 @@ static void gpxlist_view_onRowActivated(GtkTreeView *treeview,
 	gpx_t *new = NULL;
 	
 	if(g_file_test(gpx->filename, G_FILE_TEST_IS_DIR)) 
-	  new = gpx_parse_dir(dialog, gpx->filename);
+	  new = gpx_parse_dir(dialog, gpx->filename, appdata->username);
 	else
-	  new = gpx_parse(dialog, gpx->filename);
+	  new = gpx_parse(dialog, gpx->filename, appdata->username);
 	
 	if(new) {
 	  gpx_t **prev = &(appdata->gpx);
@@ -908,6 +915,28 @@ static void gpxlist_view_onRowActivated(GtkTreeView *treeview,
   }
 }
 
+/* search gpx file in gpx list */
+gboolean gpxlist_find(appdata_t *appdata, GtkTreeIter *iter, gpx_t *gpx) {
+  GtkTreeModel *model = 
+    gtk_tree_view_get_model(GTK_TREE_VIEW(appdata->gpxview));
+
+  gboolean found = 
+    gtk_tree_model_get_iter_first(model, iter);
+
+  while(found) {
+    gpx_t *this_gpx;
+    gtk_tree_model_get(model, iter, GPXLIST_COL_DATA, &this_gpx, -1);
+
+    if(gpx == this_gpx)
+      return TRUE;
+
+    found = gtk_tree_model_iter_next(model, iter);
+  }
+
+  return FALSE;
+}
+
+
 #ifndef USE_PANNABLE_AREA
 static gboolean
 view_selection_func(GtkTreeSelection *selection, GtkTreeModel *model,
@@ -1082,7 +1111,7 @@ static GtkWidget *gpxlist_create_view_and_model(appdata_t *appdata,
 }
 
 /* add last entry in gpx list to visual representation */
-static void gpxlist_add(appdata_t *appdata, gpx_t *new) {
+void gpxlist_add(appdata_t *appdata, gpx_t *new) {
   GtkTreeIter         iter;
 
   gtk_list_store_append(appdata->gpxstore, &iter);
@@ -1092,81 +1121,25 @@ static void gpxlist_add(appdata_t *appdata, gpx_t *new) {
   gpx_t **gpx = &appdata->gpx;
   while(*gpx) gpx = &((*gpx)->next);
   *gpx = new;
+
+  /* select new iter */
+  GtkTreeSelection *selection = 
+    gtk_tree_view_get_selection(GTK_TREE_VIEW(appdata->gpxview));
+  gtk_tree_selection_select_iter(selection, &iter);
+  GtkTreePath *path = 
+    gtk_tree_model_get_path(GTK_TREE_MODEL(appdata->gpxstore), &iter);
+  gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(appdata->gpxview), 
+			       path, NULL, TRUE, 0.0, 0.0);
+  gtk_tree_path_free(path);
 }
 
 /******************** end of gpxlist ********************/
 
 /******************** begin of menu *********************/
 
-typedef struct {
-  appdata_t *appdata;
-  GtkWidget *dialog;
-} about_context_t;
-
-#ifdef ENABLE_BROWSER_INTERFACE
-void on_paypal_button_clicked(GtkButton *button, about_context_t *context) {
-  gtk_dialog_response(GTK_DIALOG(context->dialog), GTK_RESPONSE_ACCEPT); 
-  browser_url(context->appdata, 
-	      "https://www.paypal.com/cgi-bin/webscr"
-	      "?cmd=_s-xclick&hosted_button_id=7400558");
-}
-#endif
-
 static void 
 cb_menu_about(GtkWidget *window, gpointer data) {
-  about_context_t context;
-
-  context.appdata = (appdata_t *)data;
-
-#ifdef ENABLE_LIBLOCATION
-  char *uses = "uses liblocation";
-#elif defined(ENABLE_GPSBT)
-  char *uses = "uses gpsbt and gpsd";
-#else
-  char *uses = "uses gpsd";
-#endif
-  
-  const gchar *authors[] = {
-    "Till Harbaum <till@harbaum.org>",
-    "John Stowers <john.stowers@gmail.com>",
-    "GCVote: Guido Wegener <guido.wegener@gmx.de>",
-    NULL };
-
-  context.dialog = g_object_new(GTK_TYPE_ABOUT_DIALOG,
-			"name", "GPXView",
-			"version", VERSION,
-			"copyright", _("Copyright 2008-2009"),
-			"authors", authors,		
-			"website", _("http://www.harbaum.org/till/maemo"),
-			"comments", _(uses),
-			NULL);
-
-#ifdef ENABLE_BROWSER_INTERFACE
-  /* add a way to donate to the project */
-  GtkWidget *alignment = gtk_alignment_new(0.5, 0, 0, 0);
-  
-  GtkWidget *hbox = gtk_hbox_new(FALSE, 8);
-  gtk_box_pack_start(GTK_BOX(hbox),
-		     gtk_label_new(_("Do you like GPXView?")), 
-		     FALSE, FALSE, 0);
-
-  GtkWidget *button = gtk_button_new();
-  gtk_button_set_image(GTK_BUTTON(button), 
-		       icon_get_widget(ICON_MISC, 5));
-  gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
-  g_signal_connect(button, "clicked", 
-		   G_CALLBACK(on_paypal_button_clicked), &context); 
-  gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
-
-  gtk_container_add(GTK_CONTAINER(alignment), hbox);
-  gtk_box_pack_start_defaults(GTK_BOX((GTK_DIALOG(context.dialog))->vbox),
-			      alignment);
-
-  gtk_widget_show_all(alignment);
-#endif
-
-  gtk_dialog_run(GTK_DIALOG(context.dialog));
-  gtk_widget_destroy(context.dialog);
+  about_box((appdata_t*)data);
 }
 
 #if defined(USE_MAEMO) && defined(HILDON_HELP)
@@ -1369,33 +1342,6 @@ typedef struct {
 } search_context_t;
 
 
-static GtkWidget *check_button_new_with_label(char *label) {
-#if !defined(USE_MAEMO) || (MAEMO_VERSION_MAJOR < 5)
-  return gtk_check_button_new_with_label(label);
-#else
-  GtkWidget *cbut = 
-    hildon_check_button_new(HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH);
-  gtk_button_set_label(GTK_BUTTON(cbut), label);
-  return cbut;
-#endif
-}
-
-static void check_button_set_active(GtkWidget *button, gboolean active) {
-#if !defined(USE_MAEMO) || (MAEMO_VERSION_MAJOR < 5)
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), active);
-#else
-  hildon_check_button_set_active(HILDON_CHECK_BUTTON(button), active);
-#endif
-}
-
-static gboolean check_button_get_active(GtkWidget *button) {
-#if !defined(USE_MAEMO) || (MAEMO_VERSION_MAJOR < 5)
-  return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button));
-#else
-  return hildon_check_button_get_active(HILDON_CHECK_BUTTON(button));
-#endif
-}
-
 static void callback_finds_toggled(GtkWidget *widget, gpointer data ) {
   search_context_t *context = (search_context_t*)data;
 
@@ -1451,12 +1397,7 @@ cb_menu_search(GtkWidget *window, gpointer data) {
 
   gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox),
 			      gtk_label_new(_("Search for:")));
-#if !defined(USE_MAEMO) || (MAEMO_VERSION_MAJOR < 5)
-  context.entry = gtk_entry_new();
-#else
-  context.entry = hildon_entry_new(HILDON_SIZE_AUTO);
-#endif
-
+  context.entry = entry_new();
   gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox),
 			      context.entry);
 
@@ -1676,6 +1617,11 @@ cb_menu_precpos(GtkWidget *window, gpointer data) {
   precise_position((appdata_t *)data);
 }
 
+static void 
+cb_menu_geotoad(GtkWidget *window, gpointer data) {
+  geotoad((appdata_t *)data);
+}
+
 #ifdef USE_STACKABLE_WINDOW
 typedef struct {
   char *label, *desc;
@@ -1790,6 +1736,8 @@ static const menu_entry_t submenu_tools_entries[] = {
     G_CALLBACK(cb_menu_geotext) },
   { "Precise Position", "Calculate a precise GPS position",  
     G_CALLBACK(cb_menu_precpos) },
+  { "GeoToad",          "Use GeoToad online downloader", 
+    G_CALLBACK(cb_menu_geotoad) },
   { NULL, NULL, NULL }
 };
 
@@ -1991,6 +1939,11 @@ void menu_create(appdata_t *appdata) {
   g_signal_connect(item, "activate", 
 		   GTK_SIGNAL_FUNC(cb_menu_precpos), appdata);
 
+  item = gtk_menu_item_new_with_label( _("GeoToad") );
+  gtk_menu_append(GTK_MENU_SHELL(submenu), item);
+  g_signal_connect(item, "activate", 
+		   GTK_SIGNAL_FUNC(cb_menu_geotoad), appdata);
+
   gtk_menu_append(GTK_MENU_SHELL(menu), gtk_separator_menu_item_new());
 
 #if defined(USE_MAEMO) && defined(HILDON_HELP)
@@ -2030,10 +1983,9 @@ void menu_create(appdata_t *appdata) {
 /********************* end of menu **********************/
 
 void cleanup(appdata_t *appdata) {
+  gconf_save_state(appdata);
+  
   gpx_free_all(appdata->gpx);
-  if(appdata->path) free(appdata->path);
-  if(appdata->image_path) free(appdata->image_path);
-  if(appdata->search_str) free(appdata->search_str);
 
 #ifdef USE_STACKABLE_WINDOW
   if(appdata->export_menu) submenu_cleanup(appdata->export_menu);
@@ -2071,7 +2023,6 @@ void cleanup(appdata_t *appdata) {
 static void on_window_destroy (GtkWidget *widget, gpointer data) {
   appdata_t *appdata = (appdata_t*)data;
 
-  gconf_save_state(appdata);
   gtk_main_quit();
   appdata->window = NULL;
 }
@@ -2362,6 +2313,8 @@ int main(int argc, char *argv[]) {
 
   gtk_init (&argc, &argv);
 
+  misc_init();
+
   curl_global_init(CURL_GLOBAL_ALL);
 
 #ifdef USE_MAEMO
