@@ -24,8 +24,6 @@
 
 /*
  * TODO: 
- * - draw caches only once
- * - prefer duplicates from selected files
  * - make semi-transparent caches selectable
  */
 
@@ -90,6 +88,9 @@ static void
 cb_map_gps(osd_button_t but, map_context_t *context) {
 
   if(but == OSD_GPS) {
+    printf("clicked OSD_GPS %p\n", context);
+    printf("clicked OSD_GPS a %p\n", context->appdata);
+
     pos_t *refpos = get_pos(context->appdata);
     if(refpos && !isnan(refpos->lat) && !isnan(refpos->lon)) {
       gint zoom;
@@ -117,17 +118,22 @@ static int dist2pixel(map_context_t *context, float km, float lat) {
 
 static gboolean map_gps_update(gpointer data) {
   map_context_t *context = (map_context_t*)data;
-  static gboolean goto_is_enabled = FALSE;
+
+#ifdef USE_MAEMO
+  if(context->appdata->goto_disable_screensaver)
+    if(osso_display_blanking_pause(context->appdata->osso_context) != OSSO_OK) 
+      fprintf(stderr, "error with display blank\n");
+#endif
 
   /* get reference position ... */
   pos_t *refpos = get_pos(context->appdata);
   gboolean ok = (refpos!= NULL) && !isnan(refpos->lat) && !isnan(refpos->lon);
 
   /* ... and enable "goto" button if it's valid */
-  if(ok != goto_is_enabled) {
+  if(ok != context->goto_is_enabled) {
     osm_gps_map_osd_enable_gps (OSM_GPS_MAP(context->widget), 
 		OSM_GPS_MAP_OSD_CALLBACK(ok?cb_map_gps:NULL), context);
-    goto_is_enabled = ok;
+    context->goto_is_enabled = ok;
   }
 
   if(ok) {
@@ -158,20 +164,41 @@ static void map_draw_cache(GtkWidget *map, cache_t *cache, gboolean semi) {
   int type = semi?ICON_CACHE_TYPE_SEMI:ICON_CACHE_TYPE;
 
   GdkPixbuf *icon = icon_get(type, cache->type);
+  GdkPixbuf *over = NULL;
 
+  if(cache->mine)                   over = icon_get(type, 14);
+  else if(cache->found)             over = icon_get(type, 12);
+  else if(cache->notes) {
+    if(cache->notes->found)         over = icon_get(type, 12);
+    else if(cache->notes->override) over = icon_get(type, 11);
+    else                            over = icon_get(type, 13);
+  }
+
+  pos_t *pos = &cache->pos;
   /* check if there's also an overwritten coordinate */
-  if(cache->notes && cache->notes->override) {
-    GdkPixbuf *over = icon_get(type, 11);
+  if(cache->notes && cache->notes->override) 
+    pos = &cache->notes->pos;
 
+  if(!isnan(pos->lat) && !isnan(pos->lon)) {
     osm_gps_map_add_image(OSM_GPS_MAP(map), 
-	  cache->notes->pos.lat, cache->notes->pos.lon, icon);
+			  pos->lat, pos->lon, icon);
+    if(over)
+      osm_gps_map_add_image(OSM_GPS_MAP(map),
+			    pos->lat, pos->lon, over);
+  }
+}
 
-    osm_gps_map_add_image(OSM_GPS_MAP(map),
-	  cache->notes->pos.lat, cache->notes->pos.lon, over);
-  } else {
-    if(!isnan(cache->pos.lat) && !isnan(cache->pos.lon))
+static void map_draw_wpt(GtkWidget *map, cache_t *cache, wpt_t *wpt) {
+  /* only draw wpts that don't equal the main point */
+  if(pos_differ(&cache->pos, &wpt->pos)) {
+    if(!isnan(wpt->pos.lat) && !isnan(wpt->pos.lon))  {
+      GdkPixbuf *icon =
+	icon_get(ICON_WPT, (wpt->sym!=WPT_SYM_UNKNOWN)?
+		 wpt->sym:WPT_SYM_REFPOINT);
+      
       osm_gps_map_add_image(OSM_GPS_MAP(map), 
-			    cache->pos.lat, cache->pos.lon, icon);
+			    wpt->pos.lat, wpt->pos.lon, icon);
+    }
   }
 }
 
@@ -195,6 +222,15 @@ static void map_draw_gpx(appdata_t *appdata, cache_display_t *caches,
       /* if nav is not given do what semi sais */
       map_draw_cache(map, cache, nav?(cache != nav):semi);
       caches[i].id = cache->id;
+
+      /* also draw waypoints of nav cache */
+      if(nav && cache == nav) {
+	wpt_t *wpt = cache->wpt;
+	while(wpt) {
+	  map_draw_wpt(map, cache, wpt);
+	  wpt = wpt->next;
+	}
+      }
     }
 
     cache = cache->next;
@@ -226,6 +262,10 @@ static void map_setup(map_context_t *context) {
     if(context->state != MAP_ALL) {
       printf("map_setup(ALL)\n");
 
+      /* clear any pending balloon */
+      context->balloon = NULL;
+      osm_gps_map_osd_clear_balloon (OSM_GPS_MAP(context->widget));
+
 #ifdef OSD_NAV
       /* no navigation in this mode */
       osm_gps_map_osd_clear_nav (OSM_GPS_MAP(context->widget));
@@ -257,12 +297,16 @@ static void map_setup(map_context_t *context) {
     if(context->state != MAP_GPX) {
       printf("map_setup(GPX)\n");
 
+      /* clear any pending balloon */
+      context->balloon = NULL;
+      osm_gps_map_osd_clear_balloon (OSM_GPS_MAP(context->widget));
+
 #ifdef OSD_NAV
       /* no navigation in this mode */
       osm_gps_map_osd_clear_nav (OSM_GPS_MAP(context->widget));
 #endif
 
-      /* clear all existing ccahe images */
+      /* clear all existing cache images */
       osm_gps_map_clear_images (OSM_GPS_MAP(context->widget));
       memset(context->caches_displayed, 0, 
 	     (cache_num+1) * sizeof(cache_display_t));
@@ -288,7 +332,7 @@ static void map_setup(map_context_t *context) {
     
     printf("map_setup(CACHE)\n");
     
-    /* no balloons in this mode */
+    /* clear any pending balloon */
     context->balloon = NULL;
     osm_gps_map_osd_clear_balloon (OSM_GPS_MAP(context->widget));
     
@@ -320,6 +364,21 @@ static void map_setup(map_context_t *context) {
 #endif
   }
 
+  /* also mark geomath position */
+
+  /* remove all existing appearances of this icon first */
+  osm_gps_map_remove_image(OSM_GPS_MAP(context->widget), 
+			   icon_get(ICON_MISC, 4));
+  
+  if(!isnan(context->appdata->geomath.lat) && 
+     !isnan(context->appdata->geomath.lon))  {
+
+    osm_gps_map_add_image(OSM_GPS_MAP(context->widget), 
+			  context->appdata->geomath.lat,
+			  context->appdata->geomath.lon,
+			  icon_get(ICON_MISC, 4));
+  }
+
   if(name) {
     char *title = g_strdup_printf(_("Map - %s"), name);
     g_free(name);
@@ -495,12 +554,18 @@ cairo_draw_pixbuf(cairo_t *cr, GdkPixbuf *buf, gint x, gint y) {
 
 #ifndef BIG_BALLOONS
 #define ICON_SIZE  ICON_CACHE_TYPE
-#define LINE_SKIP  7
 #else
 #define ICON_SIZE  ICON_CACHE_TYPE_1_5X
-#define LINE_SKIP  7
 #endif
 
+#ifndef BIG_BALLOONS
+#define FONT_SIZE 14.0
+#else
+#define FONT_SIZE 22.0
+#endif
+#define LINE_SKIP  (FONT_SIZE/4)
+
+
 static void 
 balloon_cb(osm_gps_map_balloon_event_t *event, gpointer data) {
   printf("balloon event: ");
@@ -570,25 +635,18 @@ balloon_cb(osm_gps_map_balloon_event_t *event, gpointer data) {
 			    CAIRO_FONT_SLANT_NORMAL,
 			    CAIRO_FONT_WEIGHT_NORMAL);
 
-#ifndef BIG_BALLOONS
-#define FONT_SIZE 14.0
-#else
-#define FONT_SIZE 22.0
-#endif
-
     cairo_set_font_size (event->data.draw.cr, FONT_SIZE);
     
     if(cache->name) {
       /* draw cache name */
       cairo_text_extents (event->data.draw.cr, cache->name, &extents);
-      y += extents.height;
-      cairo_move_to (event->data.draw.cr, x, y);
+      cairo_move_to (event->data.draw.cr, x, y - extents.y_bearing);
       cairo_set_source_rgba (event->data.draw.cr, 0, 0, 0, 1);
       cairo_show_text (event->data.draw.cr, cache->name);
       cairo_stroke (event->data.draw.cr);
       
       /* return to the left border and below text */
-      y += LINE_SKIP;
+      y += LINE_SKIP + FONT_SIZE;
       x = event->data.draw.rect->x;
     }
     
@@ -615,7 +673,7 @@ balloon_cb(osm_gps_map_balloon_event_t *event, gpointer data) {
       /* draw terrain stars */
       cairo_draw_pixbuf(event->data.draw.cr, icon, x, y + icon_y);
       
-      x += gdk_pixbuf_get_width(icon) + LINE_SKIP;
+      x += gdk_pixbuf_get_width(icon) + FONT_SIZE/2;
     }
     
     if(cache->difficulty) {
@@ -639,6 +697,11 @@ balloon_cb(osm_gps_map_balloon_event_t *event, gpointer data) {
       
       cairo_draw_pixbuf(event->data.draw.cr, icon, x, y + icon_y);
     }
+
+    /* draw container info */
+    /* TODO ... */
+
+
   } else if(event->type == OSM_GPS_MAP_BALLOON_EVENT_TYPE_CLICK) {
     printf("click %s event at %d %d\n", 
 	   event->data.click.down?"down":"up",
@@ -702,7 +765,7 @@ on_map_button_release_event(GtkWidget *widget,
   /* in "MAP_CACHE" state only one cache is visible */
   /* and the map is in navigation mode. the balloon is */
   /* pretty useless there */
-  if(context->press_on && (context->state != MAP_CACHE)) {
+  if(context->press_on) {
 
     coord_t coo;
     coo = osm_gps_map_get_co_ordinates(map, event->x, event->y);
@@ -765,6 +828,7 @@ static void on_window_destroy(GtkWidget *widget, map_context_t *context) {
     context->caches_displayed = NULL;
   }
 
+  printf("destroy map context\n");
   g_free(context);
   appdata->map.context = NULL;
 }
@@ -808,18 +872,26 @@ static gboolean on_focus_in(GtkWidget *widget, GdkEventFocus *event,
   return FALSE;
 }  
 
-#ifdef USE_MAEMO
 static gboolean on_focus_out(GtkWidget *widget, GdkEventFocus *event,
 			 gpointer data) {
   map_context_t *context = (map_context_t*)data;
 
   printf("map lost focus\n");
+
+  /* save new map position */
+  gfloat lat, lon;
+  g_object_get(widget, "latitude", &lat, "longitude", &lon, NULL);
+
+  context->appdata->map.pos.lat = lat;
+  context->appdata->map.pos.lon = lon;
+
+#ifdef USE_MAEMO
   gtk_timeout_remove(context->handler_id);
   context->handler_id = 0;
+#endif
 
   return FALSE;
 }
-#endif
 
 void map_update(appdata_t *appdata) {
   printf("map_update\n");
@@ -847,6 +919,8 @@ void map(appdata_t *appdata) {
   }
 
   context = appdata->map.context = g_new0(map_context_t, 1);
+  printf("allocated new context at %p\n", context);
+
   context->appdata = appdata;
   context->map_complete = FALSE;
   context->state = MAP_NONE;
@@ -914,10 +988,8 @@ void map(appdata_t *appdata) {
   g_signal_connect(G_OBJECT(context->widget), "focus-in-event", 
 		   G_CALLBACK(on_focus_in), context);
 
-#ifdef USE_MAEMO
   g_signal_connect(G_OBJECT(context->widget), "focus-out-event", 
 		   G_CALLBACK(on_focus_out), context);
-#endif
 
   g_signal_connect(G_OBJECT(context->widget), "configure-event",
 		   G_CALLBACK(on_map_configure), context);
