@@ -38,6 +38,8 @@ GtkWidget *gtk_label_small(char *str);
 GtkWidget *gtk_label_big(char *str);
 #endif
 
+gboolean pos_valid(pos_t *pos);
+
 GtkWidget *gtk_label_attrib(char *str, int size, int strikethrough);
 GtkWidget *gtk_button_attrib(char *str, int size, int strikethrough);
 void gtk_label_attrib_set(GtkWidget *label, 
@@ -59,15 +61,33 @@ char strlastchr(char *str);
 
 pos_t *get_pos(appdata_t *appdata);
 
-void distance_str(char *str, int len, float dist, gboolean mil);
-float distance_parse(char *str, gboolean imperial);
-
 GtkWidget *lat_entry_new(float lat);
 GtkWidget *lon_entry_new(float lon);
+float lat_entry_get(GtkWidget *widget);
+float lon_entry_get(GtkWidget *widget);
+void lat_entry_set(GtkWidget *widget, float lat);
+void lon_entry_set(GtkWidget *widget, float lon);
+
+void lat_label_set(GtkWidget *widget, float lat);
+void lon_label_set(GtkWidget *widget, float lon);
+void lat_label_attrib_set(GtkWidget *widget, float lat,
+			  int size, int strikethrough);
+void lon_label_attrib_set(GtkWidget *widget, float lon,
+			  int size, int strikethrough);
+
+void distance_str(char *str, int len, float dist, gboolean mil);
+float distance_parse(char *str, gboolean imperial);
 GtkWidget *dist_entry_new(float dist, gboolean mil);
-float lat_get(GtkWidget *widget);
-float lon_get(GtkWidget *widget);
-float dist_get(GtkWidget *widget, gboolean mil);
+float dist_entry_get(GtkWidget *widget, gboolean mil);
+void dist_entry_set(GtkWidget *widget, float dist, gboolean mil);
+
+void angle_str(char *str, int len, float angle); 
+float angle_parse(char *str);
+GtkWidget *angle_entry_new(float angle);
+float angle_entry_get(GtkWidget *widget);
+void angle_entry_set(GtkWidget *widget, float angle);
+
+GtkWidget *left_label_new(char *str);
 
 #ifndef USE_MAEMO
 #ifdef ENABLE_BROWSER_INTERFACE
@@ -83,4 +103,29 @@ GtkWidget *link_button_by_id(appdata_t *appdata, char *str,
 GtkWidget *link_icon_button_by_id(appdata_t *appdata, GtkWidget *icon, 
 			     const char *type, int id);
 GtkWidget *simple_text_widget(char *text);
+
+/* convenience-addons for the lat/lon entries */
+GtkWidget *preset_coordinate_picker(appdata_t *appdata, GtkWidget *lat_entry, GtkWidget *lon_entry);
+GtkWidget *goto_coordinate(appdata_t *appdata, GtkWidget *lat_entry, GtkWidget *lon_entry);
+
+GtkWidget *entry_new(void);
+
+gboolean pos_differ(pos_t *pos1, pos_t *pos2);
+gboolean pos_valid(pos_t *pos);
+
+void misc_init(void);
+
+GtkWidget *toggle_button_new_with_label(char *label);
+void toggle_button_set_active(GtkWidget *button, gboolean active);
+gboolean toggle_button_get_active(GtkWidget *button);
+
+GtkWidget *check_button_new_with_label(char *label);
+void check_button_set_active(GtkWidget *button, gboolean active);
+gboolean check_button_get_active(GtkWidget *button);
+
+GtkWidget *button_new(void);
+GtkWidget *button_new_with_label(char *label);
+
+GtkWidget *export_file(char *sel_title, char **filename);
+
 #endif // MISC_H
