@@ -82,7 +82,8 @@ typedef struct {
         cairo_surface_t *surface;
         float lat, lon;
         char *name;
-        gboolean imperial;    // display distance imperial/metric
+        gboolean imperial, mode;    // display distance imperial/metric
+        int click_sep;
     } nav;
 #endif
 
@@ -135,22 +136,12 @@ typedef struct {
 
 #define CLOSE_BUTTON_RADIUS   (BALLOON_CORNER_RADIUS)
 
-#if 0
-#define FIN   printf("entering function %s\n", __func__);
-#define FOUT  printf("leaving function %s\n", __func__);
-#else
-#define FIN   ;
-#define FOUT  ;
-#endif
-
 /* draw the bubble shape. this is used twice, once for the shape and once */
 /* for the shadow */
 static void 
 osm_gps_map_draw_balloon_shape (cairo_t *cr, int x0, int y0, int x1, int y1, 
        gboolean bottom, int px, int py, int px0, int px1) {
 
-    FIN;
-
     cairo_move_to (cr, x0, y0 + BALLOON_CORNER_RADIUS);
     cairo_arc (cr, x0 + BALLOON_CORNER_RADIUS, y0 + BALLOON_CORNER_RADIUS,
                BALLOON_CORNER_RADIUS, -M_PI, -M_PI/2);
@@ -179,14 +170,10 @@ osm_gps_map_draw_balloon_shape (cairo_t *cr, int x0, int y0, int x1, int y1,
                BALLOON_CORNER_RADIUS, M_PI/2, M_PI);
 
     cairo_close_path (cr);
-
-    FOUT;
 }
 
 static void
 osd_render_balloon(osm_gps_map_osd_t *osd) {
-    FIN;
-
     osd_priv_t *priv = (osd_priv_t*)osd->priv; 
 
     if(!priv->balloon.surface)
@@ -301,8 +288,6 @@ osd_render_balloon(osm_gps_map_osd_t *osd) {
     }
 
     cairo_destroy(cr);
-
-    FOUT;
 }
 
 /* return true if balloon is being displayed and if */
@@ -310,8 +295,6 @@ osd_render_balloon(osm_gps_map_osd_t *osd) {
 static gboolean 
 osd_balloon_check(osm_gps_map_osd_t *osd, gboolean click, gboolean down, gint x, gint y) 
 {
-    FIN;
-
     osd_priv_t *priv = (osd_priv_t*)osd->priv; 
 
     if(!priv->balloon.surface)
@@ -361,13 +344,10 @@ osd_balloon_check(osm_gps_map_osd_t *osd, gboolean click, gboolean down, gint x,
             priv->balloon.cb(&event, priv->balloon.data);
         }
     }
-    FOUT;
     return is_in;
 }
 
 void osm_gps_map_osd_clear_balloon (OsmGpsMap *map) {
-    FIN;
-
     g_return_if_fail (OSM_IS_GPS_MAP (map));
 
     osm_gps_map_osd_t *osd = osm_gps_map_osd_get(map);
@@ -383,13 +363,11 @@ void osm_gps_map_osd_clear_balloon (OsmGpsMap *map) {
         priv->balloon.lon = OSM_GPS_MAP_INVALID;
     }
     osm_gps_map_redraw(map);
-    FOUT;
 }
 
 void 
 osm_gps_map_osd_draw_balloon (OsmGpsMap *map, float latitude, float longitude, 
                               OsmGpsMapBalloonCallback cb, gpointer data) {
-    FIN;
     g_return_if_fail (OSM_IS_GPS_MAP (map));
 
     osm_gps_map_osd_t *osd = osm_gps_map_osd_get(map);
@@ -416,7 +394,6 @@ osm_gps_map_osd_draw_balloon (OsmGpsMap *map, float latitude, float longitude,
     osd_render_balloon(osd);
 
     osm_gps_map_redraw(map);
-    FOUT;
 }
 
 #endif // OSD_BALLOON
@@ -561,7 +538,6 @@ osd_shape(cairo_t *cr) {
 static gboolean
 osm_gps_map_in_circle(gint x, gint y, gint cx, gint cy, gint rad) 
 {
-    FIN;
     return( pow(cx - x, 2) + pow(cy - y, 2) < rad * rad);
 }
 
@@ -760,8 +736,6 @@ osd_source_content(osm_gps_map_osd_t *osd, cairo_t *cr, gint offset) {
 
 static void
 osd_render_source_sel(osm_gps_map_osd_t *osd, gboolean force_rerender) {
-    FIN;
-
     osd_priv_t *priv = (osd_priv_t*)osd->priv; 
 
     if(!priv->source_sel.surface ||
@@ -810,14 +784,12 @@ osd_render_source_sel(osm_gps_map_osd_t *osd, gboolean force_rerender) {
     cairo_stroke (cr);
 
     cairo_destroy(cr);
-    FOUT;
 }
 
 /* re-allocate the buffer used to draw the menu. This is used */
 /* to collapse/expand the buffer */
 static void
 osd_source_reallocate(osm_gps_map_osd_t *osd) {
-    FIN;
     osd_priv_t *priv = (osd_priv_t*)osd->priv; 
 
     /* re-allocate offscreen bitmap */
@@ -859,14 +831,12 @@ osd_source_reallocate(osm_gps_map_osd_t *osd) {
         cairo_image_surface_create(CAIRO_FORMAT_ARGB32, w+2, h+2);
 
     osd_render_source_sel(osd, TRUE);
-    FOUT;
 }
 
 #define OSD_HZ      15
 #define OSD_TIME    500
 
 static gboolean osd_source_animate(gpointer data) {
-    FIN;
     osm_gps_map_osd_t *osd = (osm_gps_map_osd_t*)data;
     osd_priv_t *priv = (osd_priv_t*)osd->priv; 
     int diff = OSD_S_EXP_W - OSD_S_W - OSD_S_X;
@@ -905,7 +875,6 @@ static gboolean osd_source_animate(gpointer data) {
     if(done) 
         priv->source_sel.handler_id = 0;
 
-    FOUT;
     return !done;
 }
 
@@ -913,7 +882,6 @@ static gboolean osd_source_animate(gpointer data) {
 static void
 osd_source_toggle(osm_gps_map_osd_t *osd)
 {
-    FIN;
     osd_priv_t *priv = (osd_priv_t*)osd->priv; 
 
     /* ignore clicks while animation is running */
@@ -939,13 +907,11 @@ osd_source_toggle(osm_gps_map_osd_t *osd)
     /* start timer to handle animation */
     priv->source_sel.handler_id = gtk_timeout_add(OSD_TIME/OSD_HZ, 
                                                   osd_source_animate, osd);
-    FOUT;
 }
 
 /* check if the user clicked inside the source selection area */
 static osd_button_t
 osd_source_check(osm_gps_map_osd_t *osd, gboolean down, gint x, gint y) {
-    FIN;
     osd_priv_t *priv = (osd_priv_t*)osd->priv; 
 
     if(!priv->source_sel.expanded)
@@ -1001,6 +967,7 @@ osd_source_check(osm_gps_map_osd_t *osd, gboolean down, gint x, gint y) {
                     
                     osd_render_source_sel(osd, TRUE);
                     osm_gps_map_repaint(OSM_GPS_MAP(osd->widget));
+                    osm_gps_map_redraw(OSM_GPS_MAP(osd->widget));
                 }
             }
             
@@ -1010,63 +977,10 @@ osd_source_check(osm_gps_map_osd_t *osd, gboolean down, gint x, gint y) {
         }
     }
 
-    FOUT;
     return OSD_NONE;
 }
 #endif // OSD_SOURCE_SEL
 
-static osd_button_t
-osd_check_int(osm_gps_map_osd_t *osd, gboolean click, gboolean down, gint x, gint y) {
-    FIN;
-    osd_button_t but = OSD_NONE;
-
-#ifdef OSD_BALLOON
-    if(down) {
-        /* needed to handle balloons that are created at click */
-        osd_priv_t *priv = (osd_priv_t*)osd->priv; 
-        priv->balloon.just_created = FALSE;
-    }
-#endif
-
-#ifdef OSD_SOURCE_SEL
-    /* the source selection area is handles internally */
-    but = osd_source_check(osd, down, x, y);
-#endif
-        
-    if(but == OSD_NONE) {
-        gint mx = x - OSD_X;
-        gint my = y - OSD_Y;
-    
-        if(OSD_X < 0)
-            mx -= (osd->widget->allocation.width - OSD_W);
-    
-        if(OSD_Y < 0)
-            my -= (osd->widget->allocation.height - OSD_H);
-    
-        /* first do a rough test for the OSD area. */
-        /* this is just to avoid an unnecessary detailed test */
-        if(mx > 0 && mx < OSD_W && my > 0 && my < OSD_H) {
-#ifndef OSD_NO_DPAD
-            but = osd_check_dpad(mx, my);
-#endif
-        }
-
-        if(but == OSD_NONE) 
-            but = osd_check_zoom(mx, my);
-    }
-
-#ifdef OSD_BALLOON
-    if(but == OSD_NONE) {
-        /* check if user clicked into balloon */
-        if(osd_balloon_check(osd, click, down, x, y)) 
-            but = OSD_BG;
-    }
-#endif
-        
-    FOUT;
-    return but;
-}
-
 #ifndef OSD_NO_DPAD
 static void
 osd_dpad_labels(cairo_t *cr, gint x, gint y) {
@@ -1207,8 +1121,6 @@ static char
 /* render a string at the given screen position */
 static int
 osd_render_centered_text(cairo_t *cr, int y, int width, char *text) {
-    FIN;
-
     if(!text) return y;
 
     char *p = g_malloc(strlen(text)+4);  // space for "...\n"
@@ -1250,14 +1162,12 @@ osd_render_centered_text(cairo_t *cr, int y, int width, char *text) {
     g_free(p);
 
     /* skip + 1/5 line */
-    FOUT;
     return y + 6*OSD_COORDINATES_FONT_SIZE/5;
 }
 
 static void
 osd_render_coordinates(osm_gps_map_osd_t *osd) 
 {
-    FIN;
     osd_priv_t *priv = (osd_priv_t*)osd->priv; 
 
     if(!priv->coordinates.surface)
@@ -1303,7 +1213,6 @@ osd_render_coordinates(osm_gps_map_osd_t *osd)
     g_free(longitude);
 
     cairo_destroy(cr);
-    FOUT;
 }
 #endif  // OSD_COORDINATES
 
@@ -1331,7 +1240,6 @@ static float get_distance(float lat1, float lon1, float lat2, float lon2) {
 static void
 osd_render_nav(osm_gps_map_osd_t *osd) 
 {
-    FIN;
     osd_priv_t *priv = (osd_priv_t*)osd->priv; 
 
     if(!priv->nav.surface || isnan(priv->nav.lat) || isnan(priv->nav.lon))
@@ -1357,10 +1265,14 @@ osd_render_nav(osm_gps_map_osd_t *osd)
     y = osd_render_centered_text(cr, y, OSD_NAV_W, priv->nav.name);
     y = osd_render_centered_text(cr, y, OSD_NAV_W, latitude);
     y = osd_render_centered_text(cr, y, OSD_NAV_W, longitude);
-    
+
     g_free(latitude);
     g_free(longitude);
 
+    /* everything below this is the compass. we need to know this */
+    /* to differ between coordinate clicks and compass clicks */
+    priv->nav.click_sep = y;
+    
     /* draw the compass */
     int radius = (OSD_NAV_H - y - 5*OSD_COORDINATES_FONT_SIZE/4)/2;
     if(radius > OSD_NAV_W/2)
@@ -1383,9 +1295,17 @@ osd_render_nav(osm_gps_map_osd_t *osd)
 #define ARROW_WIDTH     0.3
 #define ARROW_LENGTH    0.7
 
-    coord_t *gps = osm_gps_map_get_gps (OSM_GPS_MAP(osd->widget));
-    if(gps) {
-        float arot = get_bearing(gps->rlat, gps->rlon, 
+    coord_t mpos, *pos = osm_gps_map_get_gps (OSM_GPS_MAP(osd->widget));
+    if(priv->nav.mode) {
+        gfloat lat, lon;
+        g_object_get(osd->widget, "latitude", &lat, "longitude", &lon, NULL);
+        mpos.rlat = deg2rad(lat);
+        mpos.rlon = deg2rad(lon);
+        pos = &mpos;
+    }
+
+    if(pos) {
+        float arot = get_bearing(pos->rlat, pos->rlon, 
                      deg2rad(priv->nav.lat), deg2rad(priv->nav.lon));
 
         cairo_move_to(cr, 
@@ -1405,12 +1325,16 @@ osd_render_nav(osm_gps_map_osd_t *osd)
                       y + radius * -ARROW_LENGTH * -cos(arot-ARROW_WIDTH));
         
         cairo_close_path(cr);
-        cairo_set_source_rgb (cr, 0, 0, 0);
+        if(priv->nav.mode) 
+            cairo_set_source_rgb (cr, 0, 0, 0);
+        else
+            cairo_set_source_rgb (cr, 0, 0, 0.8);
+
         cairo_fill (cr);
 
         y += radius + OSD_COORDINATES_FONT_SIZE/4;
 
-        float dist = get_distance(gps->rlat, gps->rlon, 
+        float dist = get_distance(pos->rlat, pos->rlon, 
                         deg2rad(priv->nav.lat), deg2rad(priv->nav.lon));
 
         char *dist_str = NULL;
@@ -1437,11 +1361,36 @@ osd_render_nav(osm_gps_map_osd_t *osd)
     }
 
     cairo_destroy(cr);
-    FOUT;
+}
+
+/* check if the user clicked inside the source selection area */
+static osd_button_t
+osd_nav_check(osm_gps_map_osd_t *osd, gboolean down, gint x, gint y) {
+    osd_priv_t *priv = (osd_priv_t*)osd->priv; 
+    
+    if(!priv->nav.surface || down) 
+        return OSD_NONE;
+
+    x -= OSD_X;
+    if(OSD_X < 0)
+        x -= (osd->widget->allocation.width - OSD_NAV_W);
+    
+    y -= (osd->widget->allocation.height - OSD_NAV_H)/2;
+
+    if(x >= 0 && y >= 0 && x <= OSD_NAV_W && y <= OSD_NAV_H) {
+        if(y < priv->nav.click_sep)
+            osm_gps_map_set_center(OSM_GPS_MAP(osd->widget), 
+                                   priv->nav.lat, priv->nav.lon);
+        else {
+            priv->nav.mode = !priv->nav.mode;
+            osm_gps_map_redraw(OSM_GPS_MAP(osd->widget));
+        }
+    }
+
+    return OSD_NONE;
 }
 
 void osm_gps_map_osd_clear_nav (OsmGpsMap *map) {
-    FIN;
     g_return_if_fail (OSM_IS_GPS_MAP (map));
 
     osm_gps_map_osd_t *osd = osm_gps_map_osd_get(map);
@@ -1458,13 +1407,11 @@ void osm_gps_map_osd_clear_nav (OsmGpsMap *map) {
         if(priv->nav.name) g_free(priv->nav.name);
     }
     osm_gps_map_redraw(map);
-    FOUT;
 }
 
 void 
 osm_gps_map_osd_draw_nav (OsmGpsMap *map, gboolean imperial,
                           float latitude, float longitude, char *name) {
-    FIN;
     g_return_if_fail (OSM_IS_GPS_MAP (map));
 
     osm_gps_map_osd_t *osd = osm_gps_map_osd_get(map);
@@ -1488,11 +1435,67 @@ osm_gps_map_osd_draw_nav (OsmGpsMap *map, gboolean imperial,
     osd_render_nav(osd);
 
     osm_gps_map_redraw(map);
-    FOUT;
 }
 
 #endif // OSD_NAV
 
+static osd_button_t
+osd_check_int(osm_gps_map_osd_t *osd, gboolean click, gboolean down, gint x, gint y) {
+    osd_button_t but = OSD_NONE;
+
+#ifdef OSD_BALLOON
+    if(down) {
+        /* needed to handle balloons that are created at click */
+        osd_priv_t *priv = (osd_priv_t*)osd->priv; 
+        priv->balloon.just_created = FALSE;
+    }
+#endif
+
+#ifdef OSD_SOURCE_SEL
+    /* the source selection area is handles internally */
+    but = osd_source_check(osd, down, x, y);
+#endif
+
+#ifdef OSD_NAV
+    if(but == OSD_NONE) {
+        /* the source selection area is handles internally */
+        but = osd_nav_check(osd, down, x, y);
+    }
+#endif
+        
+    if(but == OSD_NONE) {
+        gint mx = x - OSD_X;
+        gint my = y - OSD_Y;
+    
+        if(OSD_X < 0)
+            mx -= (osd->widget->allocation.width - OSD_W);
+    
+        if(OSD_Y < 0)
+            my -= (osd->widget->allocation.height - OSD_H);
+    
+        /* first do a rough test for the OSD area. */
+        /* this is just to avoid an unnecessary detailed test */
+        if(mx > 0 && mx < OSD_W && my > 0 && my < OSD_H) {
+#ifndef OSD_NO_DPAD
+            but = osd_check_dpad(mx, my);
+#endif
+        }
+
+        if(but == OSD_NONE) 
+            but = osd_check_zoom(mx, my);
+    }
+
+#ifdef OSD_BALLOON
+    if(but == OSD_NONE) {
+        /* check if user clicked into balloon */
+        if(osd_balloon_check(osd, click, down, x, y)) 
+            but = OSD_BG;
+    }
+#endif
+        
+    return but;
+}
+
 
 #ifdef OSD_CROSSHAIR
 
@@ -1507,7 +1510,6 @@ osm_gps_map_osd_draw_nav (OsmGpsMap *map, gboolean imperial,
 
 static void
 osd_render_crosshair_shape(cairo_t *cr) {
-    FIN;
     cairo_arc (cr, OSD_CROSSHAIR_W/2, OSD_CROSSHAIR_H/2, 
                OSD_CROSSHAIR_RADIUS, 0,  2*M_PI);
 
@@ -1526,13 +1528,11 @@ osd_render_crosshair_shape(cairo_t *cr) {
     cairo_rel_line_to (cr, 0, OSD_CROSSHAIR_TICK);
 
     cairo_stroke (cr);
-    FOUT;
 }
 
 static void
 osd_render_crosshair(osm_gps_map_osd_t *osd) 
 {
-    FIN;
     osd_priv_t *priv = (osd_priv_t*)osd->priv; 
 
     if(!priv->crosshair.surface || priv->crosshair.rendered)
@@ -1559,7 +1559,6 @@ osd_render_crosshair(osm_gps_map_osd_t *osd)
     osd_render_crosshair_shape(cr);
 
     cairo_destroy(cr);
-    FOUT;
 }
 #endif
 
@@ -1581,7 +1580,6 @@ osd_render_crosshair(osm_gps_map_osd_t *osd)
 static void
 osd_render_scale(osm_gps_map_osd_t *osd) 
 {
-    FIN;
     osd_priv_t *priv = (osd_priv_t*)osd->priv; 
 
     if(!priv->scale.surface)
@@ -1709,14 +1707,12 @@ osd_render_scale(osm_gps_map_osd_t *osd)
     cairo_stroke(cr);
 
     cairo_destroy(cr);
-    FOUT;
 }
 #endif
 
 static void
 osd_render_controls(osm_gps_map_osd_t *osd) 
 {
-    FIN;
     osd_priv_t *priv = (osd_priv_t*)osd->priv; 
 
     if(!priv->controls.surface)
@@ -1994,13 +1990,11 @@ osd_draw(osm_gps_map_osd_t *osd, GdkDrawable *drawable)
 #endif
 
     cairo_destroy(cr);
-    FOUT;
 }
 
 static void
 osd_free(osm_gps_map_osd_t *osd) 
 {
-    FIN;
     osd_priv_t *priv = (osd_priv_t *)(osd->priv);
 
     if (priv->controls.surface)
@@ -2040,13 +2034,11 @@ osd_free(osm_gps_map_osd_t *osd)
 #endif
 
     g_free(priv);
-    FOUT;
 }
 
 static gboolean
 osd_busy(osm_gps_map_osd_t *osd) 
 {
-    FIN;
 #ifdef OSD_SOURCE_SEL
     osd_priv_t *priv = (osd_priv_t *)(osd->priv);
     return (priv->source_sel.handler_id != 0);
@@ -2057,7 +2049,6 @@ osd_busy(osm_gps_map_osd_t *osd)
 
 static osd_button_t
 osd_check(osm_gps_map_osd_t *osd, gboolean down, gint x, gint y) {
-    FIN;
     return osd_check_int(osd, TRUE, down, x, y);
 }
 
@@ -2080,18 +2071,20 @@ static osm_gps_map_osd_t osd_classic = {
 void
 osm_gps_map_osd_classic_init(OsmGpsMap *map) 
 {
-    FIN;
-    osd_priv_t *priv = osd_classic.priv = g_new0(osd_priv_t, 1);
+    osd_priv_t *priv = g_new0(osd_priv_t, 1);
+
+    /* reset entries to default value */
+    osd_classic.widget = NULL;
+    osd_classic.cb     = NULL;
+    osd_classic.data   = NULL;
+    osd_classic.priv   = priv;
 
 #ifdef OSD_BALLOON
     priv->balloon.lat = OSM_GPS_MAP_INVALID;
     priv->balloon.lon = OSM_GPS_MAP_INVALID;
 #endif
 
-    osd_classic.priv = priv;
-
     osm_gps_map_register_osd(map, &osd_classic);
-    FOUT;
 }
 
 #ifdef OSD_GPS_BUTTON
@@ -2099,7 +2092,6 @@ osm_gps_map_osd_classic_init(OsmGpsMap *map)
 /* but instead are to be used by the main application */
 void osm_gps_map_osd_enable_gps (OsmGpsMap *map, OsmGpsMapOsdCallback cb, 
                                  gpointer data) {
-    FIN;
     osm_gps_map_osd_t *osd = osm_gps_map_osd_get(map);
     g_return_if_fail (osd);
 
@@ -2111,13 +2103,11 @@ void osm_gps_map_osd_enable_gps (OsmGpsMap *map, OsmGpsMapOsdCallback cb,
     osd->render(osd);
 
     osm_gps_map_redraw(map);
-    FOUT;
 }
 #endif
 
 osd_button_t
 osm_gps_map_osd_check(OsmGpsMap *map, gint x, gint y) {
-    FIN;
     osm_gps_map_osd_t *osd = osm_gps_map_osd_get(map);
     g_return_val_if_fail (osd, OSD_NONE);
     
