@@ -29,55 +29,6 @@ typedef struct {
   GtkWidget *loc;
 } settings_dialog_state_t;
 
-/* ------------------------ special gui elements for fremantle ------------------ */
-
-static GtkWidget *toggle_button_new_with_label(char *label) {
-#if !defined(USE_MAEMO) || (MAEMO_VERSION_MAJOR < 5)
-  return gtk_check_button_new_with_label(label);
-#else
-  GtkWidget *cbut = gtk_toggle_button_new_with_label(label);
-  hildon_gtk_widget_set_theme_size(cbut, 
-	   (HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH));
-  gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(cbut), FALSE);
-  return cbut;
-#endif
-}
-
-static void toggle_button_set_active(GtkWidget *button, gboolean active) {
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), active);
-}
-
-static gboolean toggle_button_get_active(GtkWidget *button) {
-  return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button));
-}
-
-static GtkWidget *check_button_new_with_label(char *label) {
-#if !defined(USE_MAEMO) || (MAEMO_VERSION_MAJOR < 5)
-  return gtk_check_button_new_with_label(label);
-#else
-  GtkWidget *cbut = 
-    hildon_check_button_new(HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH);
-  gtk_button_set_label(GTK_BUTTON(cbut), label);
-  return cbut;
-#endif
-}
-
-static void check_button_set_active(GtkWidget *button, gboolean active) {
-#if !defined(USE_MAEMO) || (MAEMO_VERSION_MAJOR < 5)
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), active);
-#else
-  hildon_check_button_set_active(HILDON_CHECK_BUTTON(button), active);
-#endif
-}
-
-static gboolean check_button_get_active(GtkWidget *button) {
-#if !defined(USE_MAEMO) || (MAEMO_VERSION_MAJOR < 5)
-  return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button));
-#else
-  return hildon_check_button_get_active(HILDON_CHECK_BUTTON(button));
-#endif
-}
-
 /* Our usual callback function */
 static void settings_update(GtkWidget *widget, gpointer data) {
   settings_dialog_state_t *hstate = (settings_dialog_state_t *)data;
@@ -124,50 +75,6 @@ static void location_select(location_context_t *context) {
     gtk_widget_set_sensitive(context->but_remove,  FALSE);
 }
 
-#ifdef ENABLE_MAEMO_MAPPER
-#include "dbus.h"
-
-typedef struct {
-  appdata_t *appdata;
-  pos_t pos;
-  GtkWidget *import_button;
-  GtkWidget *export_button;
-  GtkWidget *lat, *lon;
-} mm_context_t;
-
-static void on_mm_import_clicked(GtkButton *button, gpointer data) {
-  char str[32];
-  mm_context_t *context = (mm_context_t*)data;
-
-  pos_lat_str(str, sizeof(str), context->appdata->mmpos.lat);
-  gtk_entry_set_text(GTK_ENTRY(context->lat), str);
-  pos_lon_str(str, sizeof(str), context->appdata->mmpos.lon);
-  gtk_entry_set_text(GTK_ENTRY(context->lon), str);
-}
-
-static void on_mm_export_clicked(GtkButton *button, gpointer data) {
-  mm_context_t *context = (mm_context_t*)data;
-
-  /* update position from entries */
-  pos_t pos;
-  pos.lat = lat_get(context->lat);
-  pos.lon = lon_get(context->lon);
-
-  g_assert(!isnan(pos.lat) && !isnan(pos.lon));
-
-  dbus_mm_set_position(context->appdata, &pos);
-}
-
-static void callback_modified_pos(GtkWidget *widget, gpointer data ) {
-  mm_context_t *context = (mm_context_t*)data;
-
-  gboolean valid =
-    (!isnan(lat_get(context->lat))) && (!isnan(lon_get(context->lon)));
-
-  gtk_widget_set_sensitive(context->export_button, valid);
-}
-#endif
-
 static void on_location_edit(GtkWidget *button, location_context_t *context) {
   GtkWidget *dialog = gtk_dialog_new_with_buttons(_("Edit Location"),
 	  GTK_WINDOW(context->settings_dialog), GTK_DIALOG_MODAL,
@@ -193,20 +100,14 @@ static void on_location_edit(GtkWidget *button, location_context_t *context) {
   } else
     printf("location edit for Home\n");
 
-#ifdef ENABLE_MAEMO_MAPPER
-  mm_context_t mm_context;
-#else
-  GtkWidget *lat, *lon;
-#endif
-
   GtkWidget *label, *name;
-  GtkWidget *table = gtk_table_new(2, 3, FALSE);
+  GtkWidget *table = gtk_table_new(3, 3, FALSE);
 
   gtk_table_attach_defaults(GTK_TABLE(table),
-		    label = gtk_label_new(_("Name:")), 0, 1, 0, 1);
+		    label = left_label_new(_("Name:")), 0, 1, 0, 1);
   gtk_misc_set_alignment(GTK_MISC(label), 1.f, 0.5f);
   gtk_table_attach_defaults(GTK_TABLE(table), 
-		    name = gtk_entry_new(), 1, 2, 0, 1);
+		    name = entry_new(), 1, 3, 0, 1);
 
   pos_t pos;
   if(loc) pos = loc->pos;
@@ -217,32 +118,23 @@ static void on_location_edit(GtkWidget *button, location_context_t *context) {
   if(isnan(pos.lat)) pos.lat = 0;
   if(isnan(pos.lon)) pos.lon = 0;
 
+  GtkWidget *latw, *lonw;
   gtk_table_attach_defaults(GTK_TABLE(table),
-		   label = gtk_label_new(_("Latitude:")), 0, 1, 1, 2);
+		   label = left_label_new(_("Latitude:")), 0, 1, 1, 2);
   gtk_misc_set_alignment(GTK_MISC(label), 1.f, 0.5f);
-#ifdef ENABLE_MAEMO_MAPPER
   gtk_table_attach_defaults(GTK_TABLE(table), 
-		    mm_context.lat = lat_entry_new(pos.lat), 1, 2, 1, 2);
-  g_signal_connect(G_OBJECT(mm_context.lat), "changed",
-		   G_CALLBACK(callback_modified_pos), &mm_context);
-#else
-  gtk_table_attach_defaults(GTK_TABLE(table), 
-		    lat = lat_entry_new(pos.lat), 1, 2, 1, 2);
-#endif
+		    latw = lat_entry_new(pos.lat), 1, 2, 1, 2);
 
-  
   gtk_table_attach_defaults(GTK_TABLE(table),
-		    label = gtk_label_new(_("Longitude:")), 0, 1, 2, 3);
+		    label = left_label_new(_("Longitude:")), 0, 1, 2, 3);
   gtk_misc_set_alignment(GTK_MISC(label), 1.f, 0.5f);
-#ifdef ENABLE_MAEMO_MAPPER
   gtk_table_attach_defaults(GTK_TABLE(table),
-		   mm_context.lon = lon_entry_new(pos.lon), 1, 2, 2, 3);
-  g_signal_connect(G_OBJECT(mm_context.lon), "changed",
-		   G_CALLBACK(callback_modified_pos), &mm_context);
-#else
+		   lonw = lon_entry_new(pos.lon), 1, 2, 2, 3);
+
   gtk_table_attach_defaults(GTK_TABLE(table),
-		   lon = lon_entry_new(pos.lon), 1, 2, 2, 3);
-#endif
+	   preset_coordinate_picker(context->appdata, latw, lonw), 2, 3, 1, 2);
+  gtk_table_attach_defaults(GTK_TABLE(table),
+	   goto_coordinate(context->appdata, latw, lonw), 2, 3, 2, 3);
 
   if(loc) 
     gtk_entry_set_text(GTK_ENTRY(name), loc->name);
@@ -251,31 +143,6 @@ static void on_location_edit(GtkWidget *button, location_context_t *context) {
     gtk_widget_set_sensitive(GTK_WIDGET(name), FALSE);
   }
 
-#ifdef ENABLE_MAEMO_MAPPER
-  mm_context.appdata = context->appdata;
-  if(loc) mm_context.pos = loc->pos;
-  else    mm_context.pos = context->appdata->home;
-
-  mm_context.import_button = gtk_button_new();
-  gtk_button_set_image(GTK_BUTTON(mm_context.import_button), 
-		       icon_get_widget(ICON_MISC, 3));
-  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->action_area),
-		    mm_context.import_button);
-  gtk_signal_connect(GTK_OBJECT(mm_context.import_button), "clicked",
-    		     (GtkSignalFunc)on_mm_import_clicked, &mm_context);
-  
-  if(!context->appdata->mmpos_valid) 
-    gtk_widget_set_sensitive(mm_context.import_button,  FALSE);
-  
-  mm_context.export_button = gtk_button_new();
-  gtk_button_set_image(GTK_BUTTON(mm_context.export_button), 
-		       icon_get_widget(ICON_MISC, 0));
-  gtk_signal_connect(GTK_OBJECT(mm_context.export_button), "clicked",
-  		     (GtkSignalFunc)on_mm_export_clicked, &mm_context);
-  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->action_area), 
-		    mm_context.export_button);
-#endif
-
   gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), table);
 
   gtk_widget_show_all(dialog);
@@ -283,13 +150,8 @@ static void on_location_edit(GtkWidget *button, location_context_t *context) {
   if(GTK_RESPONSE_ACCEPT == gtk_dialog_run(GTK_DIALOG(dialog))) {
     pos_t pos;
 
-#ifdef ENABLE_MAEMO_MAPPER
-    pos.lat = lat_get(mm_context.lat);
-    pos.lon = lon_get(mm_context.lon);
-#else
-    pos.lat = lat_get(lat);
-    pos.lon = lon_get(lon);
-#endif
+    pos.lat = lat_entry_get(latw);
+    pos.lon = lon_entry_get(lonw);
 
     if(isnan(pos.lat) || isnan(pos.lon)) 
       errorf(_("Ignoring invalid position"));
@@ -563,6 +425,7 @@ static GtkWidget *location_widget(location_context_t *context) {
   return vbox;
 }
 
+#ifdef FREMANTLE
 static GtkWidget *title_new(char *title) {
   GtkWidget *vbox = gtk_vbox_new(FALSE, 10);
   gtk_box_pack_start_defaults(GTK_BOX(vbox), gtk_label_new(""));
@@ -573,6 +436,7 @@ static GtkWidget *title_new(char *title) {
   gtk_box_pack_start_defaults(GTK_BOX(vbox), hbox);
   return vbox;
 }
+#endif
 
 void cb_menu_settings(GtkWidget *window, gpointer data) {
   appdata_t *appdata = (appdata_t *)data;
@@ -638,9 +502,7 @@ void cb_menu_settings(GtkWidget *window, gpointer data) {
 
   /* ---------------- misc old main menu entries ----------------- */
 
-#if defined(USE_MAEMO) && (MAEMO_VERSION_MAJOR >= 5)
-  // gtk_box_pack_start_defaults(GTK_BOX(vbox), title_new(_("Misc")));
-#else
+#ifndef FREMANTLE
   vbox = gtk_vbox_new(FALSE, 0);
 #endif
 
@@ -648,6 +510,18 @@ void cb_menu_settings(GtkWidget *window, gpointer data) {
   check_button_set_active(cbox_imperial, appdata->imperial);
   gtk_box_pack_start(GTK_BOX(vbox), cbox_imperial, FALSE, FALSE, 0);
 
+  hbox = gtk_hbox_new(FALSE,2);
+  gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_("Username:")), 
+		     FALSE, FALSE, 0);
+
+  GtkWidget *username = entry_new();
+  if(appdata->username)
+    gtk_entry_set_text(GTK_ENTRY(username), appdata->username);
+
+  gtk_box_pack_start(GTK_BOX(hbox), username, FALSE, FALSE, 0);
+  
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+
 #if !defined(USE_MAEMO) || (MAEMO_VERSION_MAJOR < 5)
   gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, 
 			   gtk_label_new(_("Misc")));
@@ -792,6 +666,15 @@ void cb_menu_settings(GtkWidget *window, gpointer data) {
     appdata->imperial = 
       check_button_get_active(cbox_imperial);
 
+    if(appdata->username) {
+      g_free(appdata->username);
+      appdata->username = NULL;
+    }
+
+    const char *uname = gtk_entry_get_text(GTK_ENTRY(username));
+    if(uname && strlen(uname)>0) 
+      appdata->username = g_strdup(uname);
+
     appdata->compass_damping = 0.5 + gtk_range_get_value(GTK_RANGE(scale));
 
     appdata->gpxlist_items = GPXLIST_ITEM_VALID;
